'*********************************************************
'* Audio.bi                                    6/13/2021
'*
'* Contains all of the audio related code.
'* Used by both engine and world editor
'*********************************************************




'*********************************************************
'* JukeBox
'*
'* Plays, Stops, Pauses the BGM.
'* Song - The song number you want to play
'*        -1 - Pause current song
'*        -2 - Resume a paused song
'*        -3 - Stop the song
'*********************************************************
SUB JukeBox (Song)
    IF Song <> Settings.Song AND Song <> 0 THEN
        _SNDSTOP BGM&
        Settings.Song = Song
        Settings.LoopEnabled = 1
    ELSEIF Song = Settings.Song OR Song = 0 THEN
        EXIT SUB
    END IF


    IF Song = 1 THEN
        BGM& = _SNDOPEN("Music\Overworld.mp3")
        Settings.SetLoop = 6.6
        Settings.SetLoopEnd = 39.6
        _SNDPLAY BGM&
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Playing track Overworld.", 1)

    ELSEIF Song = 2 THEN
        BGM& = _SNDOPEN("Music\Labyrinth.mp3")
        Settings.SetLoop = 0.15
        Settings.SetLoopEnd = 20.25
        _SNDPLAY BGM&
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Playing track Labyrinth.", 1)

    ELSEIF Song = 3 THEN
        BGM& = _SNDOPEN("Music\DeathMountain.mp3")
        Settings.SetLoop = 0.2
        Settings.SetLoopEnd = 23.05
        _SNDPLAY BGM&
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Playing track DeathMountain.", 1)

    ELSEIF Song = 4 THEN
        BGM& = _SNDOPEN("Music\Death.mp3")
        Settings.SetLoop = 0
        Settings.LoopEnabled = 0
        _SNDPLAY BGM&
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Playing track Death.", 1)

    ELSEIF Song = 5 THEN
        BGM& = _SNDOPEN("Music\GameOver.mp3")
        Settings.SetLoop = 0.1
        Settings.SetLoopEnd = 11.05
        _SNDPLAY BGM&
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Playing track GameOver.", 1)

    ELSEIF Song = 6 THEN
        BGM& = _SNDOPEN("Music\Triforce.mp3")
        Settings.SetLoop = 0
        Settings.LoopEnabled = 0
        _SNDPLAY BGM&
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Playing track Tri-Force.", 1)

    ELSEIF Song = 7 THEN
        BGM& = _SNDOPEN("Music\Intro.mp3")
        Settings.SetLoop = 0.1
        Settings.SetLoopEnd = 82.65
        _SNDPLAY BGM&
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Playing track Intro.", 1)

    ELSEIF Song = 8 THEN
        BGM& = _SNDOPEN("Music\ZeldaFanfare.mp3")
        Settings.SetLoop = 0
        Settings.LoopEnabled = 0
        _SNDPLAY BGM&
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Playing track Zelda's Fanfare.", 1)

    ELSEIF Song = 9 THEN
        BGM& = _SNDOPEN("Music\Ending.mp3")
        Settings.SetLoop = 0
        Settings.LoopEnabled = 0
        _SNDPLAY BGM&
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Playing track Ending.", 1)

    ELSEIF Song = 10 THEN
        BGM& = _SNDOPEN("Music\Cave.mp3")
        Settings.SetLoop = 0.1
        Settings.LoopEnabled = 1
        Settings.SetLoopEnd = 28.69
        _SNDPLAY BGM&
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Playing track Cave.", 1)


    ELSEIF Song = -1 THEN
        Settings.LoopEnabled = 0
        _SNDPAUSE BGM&
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Pausing music.", 1)

    ELSEIF Song = -2 THEN
        Settings.LoopEnabled = 1
        _SNDPLAY BGM&
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Resuming music.", 1)

    ELSEIF Song = -3 THEN
        Settings.LoopEnabled = 0
        _SNDSTOP BGM&
        BGM& = _SNDOPEN("Music\Silence.mp3")
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Audio -> Stopping music.", 1)
        Settings.Song = 0
    END IF

END SUB




'*********************************************************
'* SoundBoard
'*
'* Call to play a sound effect or ambient sound
'* SFX$ - Name of sound you want played
'* MapControl - Used by map loaded to start a ambient sound.
'*              Normally use -1
'*********************************************************
SUB SoundBoard (SFX$, MapControl)
    IF SFX$ = "Slash" THEN _SNDPLAY SFX(1)
    IF SFX$ = "SwordShoot" THEN _SNDPLAY SFX(2)
    IF SFX$ = "Hurt" THEN _SNDPLAY SFX(3)
    IF SFX$ = "Hit" THEN _SNDPLAY SFX(4)
    IF SFX$ = "Kill" THEN _SNDPLAY SFX(5)
    IF SFX$ = "Shield" THEN _SNDPLAY SFX(6)
    IF SFX$ = "BossHit" THEN _SNDPLAY SFX(7)
    IF SFX$ = "Boss3" THEN _SNDPLAY SFX(8)
    IF SFX$ = "Magic" THEN _SNDPLAY SFX(9)
    IF SFX$ = "Bomb" THEN _SNDPLAY SFX(10)
    IF SFX$ = "Expl" THEN _SNDPLAY SFX(11)
    IF SFX$ = "Flute" THEN _SNDPLAY SFX(12)
    IF SFX$ = "Text" THEN _SNDPLAY SFX(13)
    IF SFX$ = "Boomerang" THEN _SNDPLAY SFX(14)
    IF SFX$ = "Door" THEN _SNDPLAY SFX(15)
    IF SFX$ = "Secret" THEN _SNDPLAY SFX(16)
    IF SFX$ = "Heart" THEN _SNDPLAY SFX(17)
    IF SFX$ = "Candle" THEN _SNDPLAY SFX(18)
    IF SFX$ = "Die" THEN _SNDPLAY SFX(19)
    IF SFX$ = "Fanfare" THEN _SNDPLAY SFX(20)
    IF SFX$ = "LowHealth" THEN _SNDPLAY SFX(21)
    IF SFX$ = "Rupee" THEN _SNDPLAY SFX(22)
    IF SFX$ = "Item" THEN _SNDPLAY SFX(23)
    IF SFX$ = "Stairs" THEN _SNDPLAY SFX(24)
    IF SFX$ = "Flame" THEN _SNDPLAY SFX(25)
    IF SFX$ = "Switch" THEN _SNDPLAY SFX(26)
    IF SFX$ = "Key" THEN _SNDPLAY SFX(27)
    IF SFX$ = "Ganon" THEN _SNDPLAY SFX(28)
    IF SFX$ = "Refill" THEN _SNDPLAY SFX(29)

    IF MapControl = 0 THEN _SNDSTOP (AMBSFX(1)): _SNDSTOP (AMBSFX(2)): _SNDSTOP (AMBSFX(3)): _SNDSTOP (AMBSFX(4))
    IF MapControl = 1 AND _SNDPLAYING(AMBSFX(1)) = 0 THEN _SNDLOOP AMBSFX(1)
    IF MapControl = 2 AND _SNDPLAYING(AMBSFX(2)) = 0 THEN _SNDLOOP AMBSFX(2)
    IF MapControl = 3 AND _SNDPLAYING(AMBSFX(3)) = 0 THEN _SNDLOOP AMBSFX(3)
    IF MapControl = 4 AND _SNDPLAYING(AMBSFX(4)) = 0 THEN _SNDLOOP AMBSFX(4)
END SUB






'*********************************************************
'* LoadSounds
'*
'* Loads the sound effects and ambient sounds into memory.
'*********************************************************
SUB LoadSounds
    IF Settings.Debug <> 0 THEN
        CALL PrintConsole("Engine -> Loading SFX.", 1)
    ELSE
        PRINT "Engine -> Loading SFX."
    END IF

    SFX(1) = _SNDOPEN("Sounds\Slash.mp3")
    SFX(2) = _SNDOPEN("Sounds\SwordShoot.mp3")
    SFX(3) = _SNDOPEN("Sounds\Hurt.mp3")
    SFX(4) = _SNDOPEN("Sounds\Hit.mp3")
    SFX(5) = _SNDOPEN("Sounds\Kill.mp3")
    SFX(6) = _SNDOPEN("Sounds\Shield.mp3")
    SFX(7) = _SNDOPEN("Sounds\BossHit.mp3")
    SFX(8) = _SNDOPEN("Sounds\Boss3.mp3")
    SFX(9) = _SNDOPEN("Sounds\Magic.mp3")
    SFX(10) = _SNDOPEN("Sounds\Bomb.mp3")
    SFX(11) = _SNDOPEN("Sounds\Expl.mp3")
    SFX(12) = _SNDOPEN("Sounds\Flute.mp3")
    SFX(13) = _SNDOPEN("Sounds\Text.mp3")
    SFX(14) = _SNDOPEN("Sounds\Boomerang.mp3")
    SFX(15) = _SNDOPEN("Sounds\Door.mp3")
    SFX(16) = _SNDOPEN("Sounds\Secret.mp3")
    SFX(17) = _SNDOPEN("Sounds\Heart.mp3")
    SFX(18) = _SNDOPEN("Sounds\Candle.mp3")
    SFX(19) = _SNDOPEN("Sounds\Die.mp3")
    SFX(20) = _SNDOPEN("Sounds\Fanfare.mp3")
    SFX(21) = _SNDOPEN("Sounds\LowHealth.mp3")
    SFX(22) = _SNDOPEN("Sounds\Rupee.mp3")
    SFX(23) = _SNDOPEN("Sounds\Item.mp3")
    SFX(24) = _SNDOPEN("Sounds\Stairs.mp3")
    SFX(25) = _SNDOPEN("Sounds\Flame.mp3")
    SFX(26) = _SNDOPEN("Sounds\Switch.mp3")
    SFX(27) = _SNDOPEN("Sounds\Key.mp3")
    SFX(28) = _SNDOPEN("Sounds\Ganon.mp3")
    SFX(29) = _SNDOPEN("Sounds\Refill.wav")

    AMBSFX(1) = _SNDOPEN("Sounds\Boss1.mp3")
    AMBSFX(2) = _SNDOPEN("Sounds\Shore.mp3")
    AMBSFX(3) = _SNDOPEN("Sounds\Boss2.mp3")
    AMBSFX(4) = _SNDOPEN("Sounds\Boss3.mp3")
END SUB




'*********************************************************
'* MuteSounds
'*
'* Used to mute the BGM. Only used in world editor.
'*********************************************************
SUB MuteSounds
    IF Settings.Mute = 0 THEN
        Settings.Mute = 1
        _SNDVOL BGM&, 0.0
    ELSE
        Settings.Mute = 0
        _SNDVOL BGM&, 1.0
    END IF

    IF Settings.EditorActive = 1 THEN Settings.Changed = 1
END SUB


'*********************************************************
'* CheckForAudioLoop
'*
'* Checks to see if BGM is at or past its loop point then
'* loops BGM to the set start point
'*********************************************************
SUB CheckForAudioLoop
    IF Settings.LoopEnabled = 1 THEN
        ZeroTrigger = 0
        TimeTrigger = 0
        IF _SNDPLAYING(BGM&) = 0 AND Settings.SetLoop <> 0 THEN ZeroTrigger = 1
        IF _SNDGETPOS(BGM&) >= Settings.SetLoopEnd AND Settings.SetLoop <> 0 THEN TimeTrigger = 1
        IF ZeroTrigger = 1 OR TimeTrigger = 1 THEN
            IF Settings.Debug > 0 THEN CALL PrintConsole("Audio -> Loop point triggered at:" + STR$(Settings.SetLoopEnd) + " setting BGM position to:" + STR$(Settings.SetLoop), 1)
            _SNDPLAY BGM&
            _SNDSETPOS BGM&, Settings.SetLoop
        END IF
    END IF
END SUB