'************************************************************
'* Engine.bi                          12/19/2021
'*
'* Contains the main functions and subs for the game
'* Only used by game
'************************************************************


'*********************************************************
'* Sets up everything
'*
'*
'*********************************************************

SUB InitEngine
    IF Settings.Debug > 0 THEN
        CALL PrintConsole("Engine -> Initializing.", 1)
    ELSE
        PRINT "Engine -> Initializing."
    END IF

    Display& = _NEWIMAGE(Settings.RezX, Settings.RezY, 32)
    Sprites& = _NEWIMAGE(Settings.RezX, Settings.RezY, 32)
    Foreground& = _NEWIMAGE(Settings.RezX, Settings.RezY, 32)
    Buffer& = _NEWIMAGE(Settings.RezX, Settings.RezY, 32)
    Scene& = _NEWIMAGE(Settings.RezX, Settings.RezY, 32)
    Collision& = _NEWIMAGE(Settings.RezX, Settings.RezY, 32)
    UI& = _NEWIMAGE(Settings.RezX, Settings.RezY, 32)
    WaterScreen& = _NEWIMAGE(Settings.RezX, Settings.RezY, 32)
    Transition& = _NEWIMAGE(Settings.RezX, Settings.RezY, 32)
    DebugScreen& = _NEWIMAGE(Settings.RezX, Settings.RezY, 32)
    UiComposite = _NEWIMAGE(Settings.RezX, 500, 32)

    SCREEN Display&

    '_FULLSCREEN
    CALL InitTimers

    IF _FILEEXISTS("Data\Controller.inf") THEN
        CALL LoadController
    ELSE
        Controller.Enabled = 0
    END IF

    CALL LoadSprites
    CALL LoadWorldTiles
    CALL LoadMapSprites
    CALL LoadSounds
    CALL SetLootTable
    CALL LoadSave
    CALL ConnectToServer


    Settings.Alpha = 0
    'Settings.SpecialSize = 15
    Settings.CurrentZone = ActiveFile$
    Settings.LoopEnabled = 1
    Settings.UpdateUI = 1
    Settings.UiHeight = 500
    Settings.MainMenu = 1
    Settings.Intro = 1
    Settings.UiTop = (Settings.UiHeight - 200) * -1
    Settings.Font = _LOADFONT("Data\NES.ttf", 16, "MONOSPACE, BOLD")
    Settings.FontL = _LOADFONT("Data\NES.ttf", 24, "MONOSPACE, BOLD")
    Settings.FontUi = _LOADFONT("Data\NES.ttf", 20, "MONOSPACE, BOLD")
    IF RTRIM$(Settings.User) <> "EDITME" THEN Settings.UpdateBoardActive = 1

    Player1.CanMove = 1
    Water.MaxCount = 10
    ClockTrack = VAL(MID$(TIME$, 7, 2))

    IF Settings.Debug <> 0 THEN
         TIMER(FramerateTimer) ON
         'Player1.Cash = 9999
         'Player1.Triforce = 8
         'Player1.Sword = 4
         'Player1.SwordDamage = 10
         'Player1.SilverArrow = 1
         'Player1.Keys = 999
         'Player1.HaveBook = 1
         'Player1.Candle = 2
         'Player1.HaveBow = 1
         'Player1.Bombs = 999
         'Player1.HaveFlute = 1
         'Player1.HaveFood = 1
         'Player1.Invincible = 1
         'Player1.HaveMagicShield = 1
         'Player1.Health = Player1.MaxHealth
         'Player1.Boomerang = 2
         'Player1.HaveWand = 1
         'Player1.HaveRaft = 1
         'Player1.Ring = 3
         'Player1.DamageTaken = .25
         'Player1.HaveLadder = 1
         'Player1.HaveMasterKey = 1
         'Player1.Potion = 2
    END IF



    IF Settings.Debug <> 0 THEN
        CALL PrintConsole("Engine -> Done.", 1)
    ELSE
        PRINT "Engine -> Done."
    END IF
END SUB



SUB ConnectToServer
PRINT "Connecting to score server..."
ConnStatus = OpenConnection
    IF ConnStatus = 1 THEN
        PRINT "Connected to server.. Downloading current leaderboard.."
        IF GetLeaderBoard = 1 THEN
            Settings.LeaderBoardActive = 1
            CloseConnection
        ELSE
            PRINT "Could not download leaderboard.."
        END IF
    ELSEIF ConnStatus = 2 THEN
        PRINT "UserID Banned"
    ELSE
        PRINT "Could not connect to server.."
    END IF

END SUB



'*********************************************************
'* Handels input by player
'*
'*
'*********************************************************

SUB InputHandle (Action$)
    IF Settings.LockInput = 0 THEN
        IF Action$ = "North" AND Player1.LikeLocked = 0 THEN
            IF Player1.Direction <> 1 THEN Player1.AnimationDelay = Player1.MaxDelay
            Player1.Direction = 1
            Player1.Animate = 1
            Player1.YPos = Player1.YPos - Player1.MoveSpeed
        ELSEIF Action$ = "South" AND Player1.LikeLocked = 0 THEN
            IF Player1.Direction <> 3 THEN Player1.AnimationDelay = Player1.MaxDelay
            Player1.Direction = 3
            Player1.Animate = 1
            Player1.YPos = Player1.YPos + Player1.MoveSpeed
        ELSEIF Action$ = "West" AND Player1.LikeLocked = 0 THEN
            IF Player1.Direction <> 4 THEN Player1.AnimationDelay = Player1.MaxDelay
            Player1.Direction = 4
            Player1.Animate = 1
            Player1.XPos = Player1.XPos - Player1.MoveSpeed
        ELSEIF Action$ = "East" AND Player1.LikeLocked = 0 THEN
            IF Player1.Direction <> 2 THEN Player1.AnimationDelay = Player1.MaxDelay
            Player1.Direction = 2
            Player1.Animate = 1
            Player1.XPos = Player1.XPos + Player1.MoveSpeed
        ELSEIF Action$ = "Primary" AND Player1.Attacking = 0 THEN
            Player1.Attacking = 1
            CALL SpawnSprite("Sword", Player1.XPos, Player1.YPos, 0, 0, 0, 0)
        ELSEIF Action$ = "Next" THEN
            Player1.SecondaryWeapon = CycleWeapon(1)
        ELSEIF Action$ = "Prev" THEN
            Player1.SecondaryWeapon = CycleWeapon(-1)
        ELSEIF Action$ = "Secondary" AND Player1.LockSecondary = 0 AND Player1.Attacking = 0 THEN
            IF Player1.SecondaryWeapon = 1 AND Settings.PotionDelay = 0 THEN CALL BoomerangCode
            IF Player1.SecondaryWeapon = 2 AND Settings.PotionDelay = 0 AND Player1.Bombs > 0 THEN CALL SpawnSprite("Bomb", Player1.XPos, Player1.YPos, 0, 0, 0, 0)
            IF Player1.SecondaryWeapon = 3 AND Settings.PotionDelay = 0 THEN CALL ArrowCode
            IF Player1.SecondaryWeapon = 4 AND Settings.PotionDelay = 0 THEN CALL CandleCode
            IF Player1.SecondaryWeapon = 5 AND Settings.PotionDelay = 0 THEN CALL FluteCode
            IF Player1.SecondaryWeapon = 6 AND Settings.PotionDelay = 0 THEN CALL SpawnSprite("Food ", Player1.XPos, Player1.YPos, Player1.Direction, 0, 0, 1)
            IF Player1.SecondaryWeapon = 7 AND Settings.PotionDelay = 0 THEN CALL PotionCode
            IF Player1.SecondaryWeapon = 8 AND Settings.PotionDelay = 0 THEN CALL SpawnSprite("Wand", Player1.XPos, Player1.YPos, Player1.Direction, 0, 0, 1)
        END IF
    END IF
END SUB




'*********************************************************
'* Composites all the screens together after everything has been drawn
'*
'*
'*********************************************************
SUB AssembleDisplay
    IF Settings.UpdateUI = 1 THEN
        CALL DrawUI
        Settings.UpdateUI = 0
    END IF

    _DEST Buffer&
    CLS

    IF ActiveMap.Type = 1 THEN
        _PUTIMAGE , WaterScreen&, Buffer&
        _PUTIMAGE , Scene&, Buffer&
    ELSE
        _PUTIMAGE , Scene&, Buffer&
    END IF

    _PUTIMAGE , Sprites&, Buffer&

    IF ActiveMap.Type = 2 THEN _PUTIMAGE , Foreground&, Buffer&

    _PUTIMAGE , UI&, Buffer&

    IF Settings.Debug <> 0 THEN
        _DEST DebugScreen&
        LINE (Settings.LeftLimit, Settings.TopLimit)-(Settings.RightLimit, Settings.BottomLimit), _RGB(255, 0, 255), B
        LOCATE 3, 1
        PRINT "Frames Per Second:"; Settings.Frames
        PRINT "Loop Enabled:"; Settings.LoopEnabled
        PRINT _SNDPLAYING(SFX(29))
        PRINT "Zone: "; Settings.CurrentZone
        _PUTIMAGE , DebugScreen&, Buffer&
    END IF

    IF Settings.GameOver = 1 THEN
        _FONT Settings.FontL
        LOCATE 20, 15
        COLOR _RGB(255, 255, 255)
        PRINT "GAME OVER"
    END IF

    _PUTIMAGE , Buffer&, Display&
END SUB



'*********************************************************
'* Drops loot based on loot table
'*
'*
'*********************************************************

SUB DropLoot (MobNum)
    IF Sprites(MobNum).LootCategory > 0 AND PercentChance(50) = 1 THEN
        IF LootTable(Sprites(MobNum).LootCategory, Settings.KillCount) = 1 THEN CALL SpawnSprite("Rupee", Sprites(MobNum).XPos, Sprites(MobNum).YPos, 0, 0, 0, 0)
        IF LootTable(Sprites(MobNum).LootCategory, Settings.KillCount) = 2 THEN CALL SpawnSprite("P-Bmb", Sprites(MobNum).XPos, Sprites(MobNum).YPos, 0, 0, 0, 0)
        IF LootTable(Sprites(MobNum).LootCategory, Settings.KillCount) = 3 THEN CALL SpawnSprite("Heart", Sprites(MobNum).XPos, Sprites(MobNum).YPos, 0, 0, 0, 0)
        IF LootTable(Sprites(MobNum).LootCategory, Settings.KillCount) = 4 THEN CALL SpawnSprite("Fairy", Sprites(MobNum).XPos, Sprites(MobNum).YPos, 0, 0, 0, 0)
        IF LootTable(Sprites(MobNum).LootCategory, Settings.KillCount) = 5 THEN CALL SpawnSprite("Clock", Sprites(MobNum).XPos, Sprites(MobNum).YPos, 0, 0, 0, 0)
        IF LootTable(Sprites(MobNum).LootCategory, Settings.KillCount) = 6 THEN CALL SpawnSprite("5Rupe", Sprites(MobNum).XPos, Sprites(MobNum).YPos, 0, 0, 0, 0)
    END IF
END SUB



'*********************************************************
'* Sets Link's start position. Sprite #1 pr 'Flag'
'* Matrix selects either transition matrix or active.
'*
'*********************************************************
SUB SetStartPos (Flag, Matrix)
    Player1.XPos = Settings.RezX / 2
    Player1.YPos = Settings.RezY / 2

    IF Matrix = 1 THEN
        SpriteLimitX = ActiveMap.SpriteLimitX
        SpriteLimitY = ActiveMap.SpriteLimitY
    ELSE
        SpriteLimitX = TransMap.SpriteLimitX
        SpriteLimitY = TransMap.SpriteLimitY
    END IF

    FOR y = 1 TO SpriteLimitY
        FOR x = 1 TO SpriteLimitX
            IF Matrix = 1 THEN
                IF SpriteMatrix(x, y) = Flag OR SpriteMatrix(x, y) = 1 THEN
                    Player1.XPos = Settings.LeftLimit + ((x - 1) * Settings.SquareWidth) + (Settings.SquareWidth / 2)
                    Player1.YPos = Settings.TopLimit + ((y - 1) * Settings.SquareHeight) + (Settings.SquareHeight / 2)
                    IF Settings.Debug = 3 THEN CALL PrintConsole("World -> Setting player position to: " + STR$(Player1.XPos) + ":" + STR$(Player1.YPos), 3)

                    IF ActiveMap.Type = 5 THEN Player1.XPos = Player1.XPos + 5
                    IF SpriteMatrix(x, y) <> 1 THEN EXIT SUB
                END IF
            ELSE
                IF TransMatrix(x, y) = Flag OR TransMatrix(x, y) = 1 THEN
                    Player1.XPos = Settings.LeftLimit + ((x - 1) * Settings.SquareWidth) + (Settings.SquareWidth / 2)
                    Player1.YPos = Settings.TopLimit + ((y - 1) * Settings.SquareHeight) + (Settings.SquareHeight / 2)
                    IF Settings.Debug = 3 THEN CALL PrintConsole("World -> Setting player position to: " + STR$(Player1.XPos) + ":" + STR$(Player1.YPos), 3)

                    IF TransMap.Type = 5 THEN Player1.XPos = Player1.XPos + 5
                    IF TransMatrix(x, y) <> 1 THEN EXIT SUB
                END IF
            END IF
        NEXT x
    NEXT y
END SUB




'*********************************************************
'* Supplied by bPlus on the QB64 forums to convert an image
'* to black and white. Used in link's death sequence
'*
'*********************************************************
FUNCTION Convert2BW (imageh&)
    'supplied by bplus

    storeDest = _DEST
    iw& = _WIDTH(imageh&): ih& = _HEIGHT(imageh&)
    newh& = _NEWIMAGE(iw&, ih&, 32)
    _SOURCE imageh&
    _DEST newh&
    FOR y = 0 TO ih& - 1
        FOR x = 0 TO iw& - 1
            cAnalysis POINT(x, y), rr, gg, bb, aa
            mix = (rr + gg + bb - 10) / 3
            LINE (x, y)-STEP(0, 0), _RGB32(mix, mix, mix, aa), BF
        NEXT
    NEXT

    Convert2BW = _COPYIMAGE(newh&)

    _FREEIMAGE imageh&
    _FREEIMAGE newh&
END FUNCTION


SUB cAnalysis (c AS _UNSIGNED LONG, outRed, outGrn, outBlu, outAlp)
    outRed = _RED32(c): outGrn = _GREEN32(c): outBlu = _BLUE32(c): outAlp = _ALPHA32(c)
END SUB




'*********************************************************
'* Modified version of above function to fade to black
'*
'*
'*********************************************************
FUNCTION Fade2Black (imageh&)
    storeDest = _DEST
    iw& = _WIDTH(imageh&): ih& = _HEIGHT(imageh&)
    newh& = _NEWIMAGE(iw&, ih&, 32)
    _SOURCE imageh&
    _DEST newh&
    FOR y = 0 TO ih& - 1
        FOR x = 0 TO iw& - 1
            cAnalysis POINT(x, y), rr, gg, bb, aa
            mix = (rr + gg + bb - 10) / 4
            LINE (x, y)-STEP(0, 0), _RGB32(mix, mix, mix, aa), BF
        NEXT
    NEXT

    Fade2Black = _COPYIMAGE(newh&)

    _FREEIMAGE imageh&
    _FREEIMAGE newh&
END FUNCTION




'*********************************************************
'* Draws all the sprites currently active
'*
'*
'*********************************************************

SUB DrawSprites
    _DEST DebugScreen&
    CLS
    _CLEARCOLOR _RGB(0, 0, 0)
    _DEST Collision&
    CLS
    _DEST Sprites&
    CLS
    _CLEARCOLOR _RGB(0, 0, 0)

    CALL DrawWalls
    CALL DrawDoorPath
    IF Player1.Hurt = 0 AND Player1.OnRaft = 0 AND Player1.Invincible < 2 AND Player1.Frame < 4 THEN CALL DrawPlayer
    CALL DrawSpecial
    IF Settings.BlockTrigger <> 0 THEN CALL DrawBlock
    IF Attack(3).Active = 1 AND Player1.SecondaryWeapon = 2 THEN CALL BombCode
    CALL DrawMobs
    CALL DrawZoneWalls
    IF Player1.Invincible < 2 THEN CALL CheckPlayerCollision
    CALL AttackHandle
    CALL DrawPlayerSprite(Sprites&)
    CALL DrawProjectiles
END SUB



'*********************************************************
'* ReplaceSprite
'*
'* Replaces a sprite with another, permanently
'*********************************************************

SUB ReplaceSprite (ReplaceNum, ReplaceWith, DontSave)
    IF Settings.Debug > 2 THEN CALL PrintConsole("Engine -> Replacing Sprite# :" + STR$(ReplaceNum) + " with " + STR$(ReplaceWith), 3)

    FOR y = 1 TO ActiveMap.SpriteLimitY
        FOR x = 1 TO ActiveMap.SpriteLimitX
            IF SpriteMatrix(x, y) = ReplaceNum THEN
                SpriteMatrix(x, y) = ReplaceWith
                IF DontSave = 0 THEN CALL SaveMap
            END IF
        NEXT x
    NEXT y
END SUB


'*********************************************************
'* Gets matrix 'x' position based on x position on screen
'*
'*
'*********************************************************

FUNCTION GetMatrixX (SpriteNum, Matrix)
    IF Matrix = 1 THEN
        XPos = Sprites(SpriteNum).XPos
    ELSEIF Matrix = 2 THEN
        XPos = Projectile(SpriteNum).XPos
    ELSE
        XPos = Special(SpriteNum).XPos
    END IF

    SpriteWidth = (Settings.SquareWidth / 2) - 1
    XPos = XPos - Settings.LeftLimit
    XPos = XPos - SpriteWidth
    XPos = XPos / Settings.SquareWidth

    GetMatrixX = XPos + 1
END FUNCTION



'*********************************************************
'* Gets matrix 'y' position based on y position on screen
'*
'*
'*********************************************************

FUNCTION GetMatrixY (SpriteNum, Matrix)
    IF Matrix = 1 THEN
        YPos = Sprites(SpriteNum).YPos
    ELSEIF Matrix = 2 THEN
        YPos = Projectile(SpriteNum).YPos
    ELSE
        YPos = Special(SpriteNum).YPos
    END IF

    SpriteHeight = (Settings.SquareHeight / 2) - 1
    YPos = YPos - Settings.TopLimit
    YPos = YPos - SpriteHeight
    YPos = YPos / Settings.SquareHeight

    GetMatrixY = YPos + 1
END FUNCTION




'*********************************************************
'* DeleteSprite
'*
'* Deletes sprite from existence
'*********************************************************

SUB DeleteSprite (SpriteNum)
    IF Settings.Debug > 2 THEN CALL PrintConsole("Engine -> Removing Sprite# :" + STR$(SpriteNum) + " from world", 3)

    FOR y = 1 TO ActiveMap.SpriteLimitY
        FOR x = 1 TO ActiveMap.SpriteLimitX
            IF SpriteMatrix(x, y) = SpriteNum THEN
                SpriteMatrix(x, y) = 0
                CALL SaveMap
                CALL SaveGame("DeleteSprite")
                EXIT SUB
            END IF
        NEXT x
    NEXT y
END SUB




'*********************************************************
'* BossDeath
'*
'* Deletes a boss from existence when defeated
'*********************************************************
SUB BossDeath (SpawnName$)
    IF SpawnName$ = "Aqua " THEN CALL DeleteSprite(38)
    IF SpawnName$ = "Dodon" THEN CALL DeleteSprite(39)
    IF SpawnName$ = "ManhM" THEN CALL DeleteSprite(40)
    IF SpawnName$ = "GleeM" THEN CALL DeleteSprite(41)
    IF SpawnName$ = "Digdo" THEN CALL DeleteSprite(42)
    IF SpawnName$ = "RGohm" THEN CALL DeleteSprite(43)
    IF SpawnName$ = "BGohm" THEN CALL DeleteSprite(44)
    IF SpawnName$ = "Ganno" THEN CALL DeleteSprite(45)
END SUB








'************************************************************
'* SUB InitTimers
'*
'* Creates and sets up the timers
'************************************************************
SUB InitTimers
    SwordDelayTimer = _FREETIMER
    FramerateTimer = _FREETIMER
    WizzFireTimer = _FREETIMER
    BubbleTimer = _FREETIMER
    CashTimer = _FREETIMER
    PotionTimer = _FREETIMER
    GameOverTimer = _FREETIMER

    ON TIMER(SwordDelayTimer, .25) ResetSword
    ON TIMER(FramerateTimer, 1) PostFramerate
    ON TIMER(WizzFireTimer, .5) WizzFire
    ON TIMER(BubbleTimer, 5) UnlockSword
    ON TIMER(CashTimer, .07) ModifyCash
    ON TIMER(PotionTimer, 1) PotionReset
    ON TIMER(GameOverTimer, 3) GameOver
END SUB





'************************************************************
'* FUNCTION IfAlreadyCleared
'*
'* Returns 1 if zone was already cleared. Uses the cleared array
'************************************************************
FUNCTION IfAlreadyCleared
    IfAlreadyCleared = 0
    FOR x = 1 TO Settings.ClearedArraySize
        IF Settings.CurrentZone = ClearedArray(x) THEN
            IfAlreadyCleared = 1
            IF Settings.Debug <> 0 THEN CALL PrintConsole("World -> Zone on cleared list, ignoring spawn matrix.", 1)
        END IF
    NEXT x
END FUNCTION




'************************************************************
'* FUNCTION AreAllDead
'*
'* Returns 1 if all mobs are dead. 0 otherwise
'************************************************************
FUNCTION AreAllDead
    AreAllDead = 1
    FOR x = 1 TO Settings.MatrixSize
        IF Sprites(x).Active = 1 AND Sprites(x).HardCount = 1 THEN
            AreAllDead = 0
            EXIT FOR
        END IF
    NEXT x
    IF ActiveMap.Type = 3 THEN AreAllDead = 0
END FUNCTION



'************************************************************
'* FUNCTION LinkInRoom
'*
'* Returns 1 if Link is inside a dungeon room, used to lock the doors.
'************************************************************
FUNCTION LinkInRoom
    LinkInRoom = 0
    SizeLimit = Player1.Size * .8
    IF Player1.XPos > Settings.LeftLimit + SizeLimit AND Player1.XPos < Settings.RightLimit - SizeLimit AND Player1.YPos > Settings.TopLimit + SizeLimit AND Player1.YPos < Settings.BottomLimit - SizeLimit THEN LinkInRoom = 1
END FUNCTION



'************************************************************
'* Function EmptySnake
'*
'* Checks the 4 Snake trackers and returns the number of the one not being used.
'************************************************************
FUNCTION EmptySnake
    EmptySnake = 0
    FOR x = 1 TO 4
        IF Snakes(x) = 0 THEN
            EmptySnake = x
            EXIT FOR
        END IF
    NEXT x
    IF EmptySnake = 0 THEN ERROR 96
END FUNCTION




'************************************************************
'* SUB AddToCleared
'*
'* When zone has been cleared, this is used to add zone to the no-spawn list
'************************************************************
SUB AddToCleared
    IF Player1.Death = 0 AND AlreadyOnClearList = 0 THEN
        Settings.ClearedArrayLocation = Settings.ClearedArrayLocation + 1
        IF Settings.ClearedArrayLocation > Settings.ClearedArraySize THEN Settings.ClearedArrayLocation = 1
        ClearedArray(Settings.ClearedArrayLocation) = Settings.CurrentZone
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Engine -> Added map: " + Settings.CurrentZone + " to cleared list.", 1)
    END IF
END SUB



'************************************************************
'* SUB ResetSword
'*
'* Called when SwordDelayTimer has gone off to reset the timer
'* so Link can attack again, otherwise he would rapid attack
'************************************************************
SUB ResetSword
    Settings.LockAttack = 0
    TIMER(SwordDelayTimer) OFF
END SUB




'************************************************************
'* SUB CenterLink
'*
'* Used to center Link on the current square he is on.
'************************************************************
SUB CenterLink
    SectionX = INT((Player1.XPos - Settings.LeftLimit) / Settings.SquareWidth)
    SectionY = INT(((Player1.YPos + 15) - Settings.TopLimit) / Settings.SquareHeight)

    Player1.XPos = ((Settings.SquareWidth * SectionX) + Settings.LeftLimit) + Player1.Size
    Player1.YPos = (Settings.SquareHeight * SectionY) + Settings.TopLimit - (Player1.Size / 2)
END SUB



'************************************************************
'* FUNCTION EmptySprite
'*
'* Used to find an unused location in the sprite matrix.
'* If not game will error out with code 98.
'************************************************************
FUNCTION EmptySprite
    EmptySprite = 0
    FOR x = 1 TO Settings.MatrixSize
        IF Sprites(x).Active = 0 THEN
            EmptySprite = x
            EXIT FOR
        END IF
    NEXT x
    IF EmptySprite = 0 THEN ERROR 98
END FUNCTION



'************************************************************
'* FUNCTION EmptyProjectile
'*
'* Used to find an unused location in the projectile matrix.
'* If not game will error out with code 93.
'************************************************************
FUNCTION EmptyProjectile (Reverse)
    EmptyProjectile = 0
    IF Reverse = 1 THEN
        FOR x = Settings.MatrixSize TO 1 STEP -1
            IF Projectile(x).Active = 0 THEN
                EmptyProjectile = x
                EXIT FOR
            END IF
        NEXT x
    ELSE
        FOR x = 1 TO Settings.MatrixSize
            IF Projectile(x).Active = 0 THEN
                EmptyProjectile = x
                EXIT FOR
            END IF
        NEXT x
    END IF
    IF EmptyProjectile = 0 THEN ERROR 93
END FUNCTION





'************************************************************
'* FUNCTION EmptySpecial
'*
'* Used to find an unused location in the special sprite matrix.
'* If not game will error out with code 94.
'************************************************************
FUNCTION EmptySpecial (Reverse)
    EmptySpecial = 0
    IF Reverse = 1 THEN
        FOR x = Settings.MatrixSize TO 1 STEP -1
            IF Special(x).Active = 0 THEN
                EmptySpecial = x
                EXIT FOR
            END IF
        NEXT x
    ELSE
        FOR x = 1 TO Settings.MatrixSize
            IF Special(x).Active = 0 THEN
                EmptySpecial = x
                EXIT FOR
            END IF
        NEXT x
    END IF
    IF EmptySpecial = 0 THEN ERROR 94
END FUNCTION




'************************************************************
'* SUB ModifyCash
'*
'* Used to modify the rupies one at a time.
'* Called when Settings.ModifyCash <> 0
'************************************************************
SUB ModifyCash
    IF Settings.ModifyCash < 0 THEN
        Player1.Cash = Player1.Cash - 1
        Settings.ModifyCash = Settings.ModifyCash + 1
        CALL SoundBoard("Text", -1)
        Settings.UpdateUI = 1
    ELSE
        Player1.Cash = Player1.Cash + 1
        Settings.ModifyCash = Settings.ModifyCash - 1
        CALL SoundBoard("Text", -1)
        Settings.UpdateUI = 1
    END IF
END SUB


'************************************************************
'* SUB PostFramerate
'*
'* Used to update framerate count. Only ran when debug mode
'* is active
'************************************************************
SUB PostFramerate
    Settings.Frames = Framecount
    Framecount = 0
END SUB




'************************************************************
'* FUNCTION ThrowBack
'*
'* Used to flip an objects direction, usually when an object
'* has taken damage
'************************************************************
FUNCTION ThrowBack (CurrentDirection)
    IF CurrentDirection = 1 THEN ThrowBack = 3
    IF CurrentDirection = 2 THEN ThrowBack = 4
    IF CurrentDirection = 3 THEN ThrowBack = 1
    IF CurrentDirection = 4 THEN ThrowBack = 2
END FUNCTION




'************************************************************
'* SUB SetLootTable
'*
'* Builds the loot table for the game. Google Zelda1 Loot table.
'************************************************************
SUB SetLootTable
    IF Settings.Debug <> 0 THEN
        CALL PrintConsole("Engine -> Building loot table.", 1)
    ELSE
        PRINT "Engine -> Building loot table."
    END IF

    LootTable(1, 1) = 1
    LootTable(2, 1) = 2
    LootTable(3, 1) = 1
    LootTable(4, 1) = 3

    LootTable(1, 2) = 3
    LootTable(2, 2) = 1
    LootTable(3, 2) = 3
    LootTable(4, 2) = 4

    LootTable(1, 3) = 1
    LootTable(2, 3) = 5
    LootTable(3, 3) = 1
    LootTable(4, 3) = 1

    LootTable(1, 4) = 4
    LootTable(2, 4) = 1
    LootTable(3, 4) = 6
    LootTable(4, 4) = 3

    LootTable(1, 5) = 1
    LootTable(2, 5) = 3
    LootTable(3, 5) = 3
    LootTable(4, 5) = 4

    LootTable(1, 6) = 3
    LootTable(2, 6) = 2
    LootTable(3, 6) = 5
    LootTable(4, 6) = 3

    LootTable(1, 7) = 3
    LootTable(2, 7) = 1
    LootTable(3, 7) = 1
    LootTable(4, 7) = 3

    LootTable(1, 8) = 1
    LootTable(2, 8) = 2
    LootTable(3, 8) = 1
    LootTable(4, 8) = 3

    LootTable(1, 9) = 1
    LootTable(2, 9) = 3
    LootTable(3, 9) = 1
    LootTable(4, 9) = 1

    LootTable(1, 10) = 3
    LootTable(2, 10) = 3
    LootTable(3, 10) = 6
    LootTable(4, 10) = 3
END SUB



'************************************************************
'* SUB SaveGame
'*
'* Saves the game. Source$ is used for debugging.
'************************************************************
SUB SaveGame (Source$)
    IF Settings.Debug > 0 THEN CALL PrintConsole("Engine -> Saving game, called from: " + Source$, 1)

    OPEN "Data\Save.mss" FOR RANDOM AS #1
    PUT #1, , Player1
    CLOSE #1
END SUB


'************************************************************
'* SUB LoadSave
'*
'* Loads the save file
'************************************************************
SUB LoadSave
    IF _FILEEXISTS("Data\Save.mss") THEN
        OPEN "Data\Save.mss" FOR RANDOM AS #1
            GET #1, , Player1
        CLOSE #1
    ELSE
        Player1.Bombs = -1
        Player1.SecondaryWeapon = 0
    END IF
END SUB


'*********************************************************
'* Used in dungeons. Used to clear the walls when doors
'* are opened or bomb exploded.
'*
'*********************************************************
SUB DrawDoorPath
    _DEST Collision&

    IF ActiveMap.Type = 5 THEN
        IF RTRIM$(ActiveMap.NFile) <> "NULL" THEN LINE (0, Settings.TopLimit - 10)-(Settings.RezX / 2, Settings.TopLimit), _RGB(10, 0, 1), BF
        IF RTRIM$(ActiveMap.EFile) <> "NULL" THEN LINE (Settings.RezX / 2, Settings.TopLimit - 10)-(Settings.RezX, Settings.TopLimit), _RGB(10, 50, 2), BF

        LINE (0, Settings.TopLimit)-(190, Settings.RezY), _RGB(20, 0, 0), BF
        LINE (0, 652)-(Settings.RezX, Settings.RezY), _RGB(20, 0, 0), BF
        LINE (252, 480)-(704, 604), _RGB(20, 0, 0), BF
        LINE (768, Settings.TopLimit)-(Settings.RezX, Settings.RezY), _RGB(20, 0, 0), BF

        IF ActiveMap.SubType = 1 THEN
            LINE (252, Settings.TopLimit)-(448, 604), _RGB(20, 0, 0), BF
            LINE (252, Settings.TopLimit)-(Settings.RezX, 428), _RGB(20, 0, 0), BF
        ELSE
            LINE (252, Settings.TopLimit)-(704, 604), _RGB(20, 0, 0), BF
        END IF
    END IF

    'look for zone b=5 is with stair animation and b=6 is without
    FOR y = 1 TO ActiveMap.SpriteLimitY
        FOR x = 1 TO ActiveMap.SpriteLimitX
            IF SpriteMatrix(x, y) = 31 + Settings.SpriteDivider THEN
                SpriteWidth = (Settings.SquareWidth / 2) - 1
                SpriteHeight = (Settings.SquareHeight / 5) - 1
                SpriteX = (Settings.SquareWidth * (x - 1)) + Settings.LeftLimit + SpriteWidth
                Spritey = (Settings.SquareHeight * (y - 1)) + Settings.TopLimit + SpriteHeight

                LINE (SpriteX - SpriteWidth - 1, Spritey - SpriteHeight - 1)-(SpriteX + SpriteWidth + 1, Spritey + SpriteHeight + 1), _RGB(10, 50, 5), BF
            ELSEIF SpriteMatrix(x, y) = 65 + Settings.SpriteDivider THEN
                SpriteWidth = (Settings.SquareWidth / 2) - 1
                SpriteHeight = (Settings.SquareHeight / 2) - 1
                SpriteX = (Settings.SquareWidth * (x - 1)) + Settings.LeftLimit + SpriteWidth
                Spritey = (Settings.SquareHeight * (y - 1)) + Settings.TopLimit + SpriteHeight

                LINE (SpriteX - SpriteWidth - 2, Spritey - SpriteHeight - 2)-(SpriteX + SpriteWidth + 2, Spritey + SpriteHeight + 2), _RGB(10, 50, 6), BF
            END IF
        NEXT x
    NEXT y
END SUB



'*********************************************************
'* Draws the zone walls, these areas will start a transition
'* when player collides with them.
'*
'*********************************************************
SUB DrawZoneWalls
    _DEST Collision&

    IF ActiveMap.Type = 1 OR ActiveMap.Type = 3 OR ActiveMap.Type = 4 THEN
        IF RTRIM$(ActiveMap.NFile) <> "NULL" THEN LINE (Settings.LeftLimit, Settings.TopLimit - 10)-(Settings.RightLimit, Settings.TopLimit), _RGB(10, 0, 1), BF
        IF RTRIM$(ActiveMap.EFile) <> "NULL" THEN LINE (Settings.RightLimit - 5, Settings.TopLimit - 5)-(Settings.RightLimit, Settings.BottomLimit), _RGB(10, 0, 2), BF
        IF RTRIM$(ActiveMap.SFile) <> "NULL" THEN LINE (Settings.LeftLimit, Settings.BottomLimit - 10)-(Settings.RightLimit, Settings.BottomLimit), _RGB(10, 0, 3), BF
        IF RTRIM$(ActiveMap.WFile) <> "NULL" THEN LINE (Settings.LeftLimit, Settings.TopLimit)-(Settings.LeftLimit + 5, Settings.BottomLimit), _RGB(10, 0, 4), BF

    ELSEIF ActiveMap.Type = 2 THEN
        IF ActiveMap.DoorN = 1 OR ActiveMap.DoorN > 4 THEN LINE (473, 208)-(543, 306), _RGB(0, 0, 0), BF
        IF ActiveMap.DoorE = 1 OR ActiveMap.DoorE > 4 THEN LINE (892, 460)-(990, 516), _RGB(0, 0, 0), BF
        IF ActiveMap.DoorS = 1 OR ActiveMap.DoorS > 4 THEN LINE (473, 664)-(543, 743), _RGB(0, 0, 0), BF
        IF ActiveMap.DoorW = 1 OR ActiveMap.DoorW > 4 THEN LINE (25, 460)-(125, 516), _RGB(0, 0, 0), BF

        IF ActiveMap.DoorN = 2 THEN LINE (473, 208)-(543, 306), _RGB(20, 10, 1), BF
        IF ActiveMap.DoorE = 2 THEN LINE (892, 460)-(990, 516), _RGB(20, 10, 2), BF
        IF ActiveMap.DoorS = 2 THEN LINE (473, 664)-(543, 743), _RGB(20, 10, 3), BF
        IF ActiveMap.DoorW = 2 THEN LINE (25, 460)-(125, 516), _RGB(20, 10, 4), BF

        IF ActiveMap.DoorN = 3 AND Settings.LockDown < 1 THEN LINE (473, 208)-(543, 306), _RGB(0, 0, 0), BF
        IF ActiveMap.DoorE = 3 AND Settings.LockDown < 1 THEN LINE (892, 460)-(990, 516), _RGB(0, 0, 0), BF
        IF ActiveMap.DoorS = 3 AND Settings.LockDown < 1 THEN LINE (473, 664)-(543, 743), _RGB(0, 0, 0), BF
        IF ActiveMap.DoorW = 3 AND Settings.LockDown < 1 THEN LINE (25, 460)-(125, 516), _RGB(0, 0, 0), BF

        IF ActiveMap.DoorN = 7 AND Player1.InRoom = 1 THEN LINE (473, 208)-(543, 306), _RGB(20, 0, 20), BF
        IF ActiveMap.DoorE = 7 AND Player1.InRoom = 1 THEN LINE (892, 460)-(990, 516), _RGB(20, 0, 20), BF
        IF ActiveMap.DoorS = 7 AND Player1.InRoom = 1 THEN LINE (473, 664)-(543, 743), _RGB(20, 0, 20), BF
        IF ActiveMap.DoorW = 7 AND Player1.InRoom = 1 THEN LINE (25, 460)-(125, 516), _RGB(20, 0, 20), BF

        IF ActiveMap.DoorN > 0 THEN LINE (450, 200)-(575, 208), _RGB(10, 0, 1), BF
        IF ActiveMap.DoorE > 0 THEN LINE (1000, 418)-(1010, 530), _RGB(10, 0, 2), BF
        IF ActiveMap.DoorS > 0 THEN LINE (450, 743)-(575, 750), _RGB(10, 0, 3), BF
        IF ActiveMap.DoorW > 0 THEN LINE (15, 418)-(25, 530), _RGB(10, 0, 4), BF
    END IF
END SUB


'*********************************************************
'* Prints text on the screen using the NES font
'*
'*
'*********************************************************
SUB PrintTxT (TextToPrint$)
    IF LEN(TextToPrint$) > Settings.MaxChar THEN ERROR 92
    LineY = 20

    DO
        IF LEFT$(TextToPrint$, 1) = CHR$(13) THEN
            LineY = LineY + 1
            TextToPrint$ = MID$(TextToPrint$, 2, LEN(TextToPrint$))
        ELSE
            EXIT DO
        END IF
    LOOP

    COLOR _RGB(255, 255, 255)
    CenterPos = (INT(Settings.MaxChar / 2) - INT(LEN(TextToPrint$) / 2)) + 7

    _DEST Scene&
    _FONT Settings.Font
    LOCATE LineY, CenterPos
    PRINT TextToPrint$
    _DEST Display&
    _FONT Settings.Font
    LOCATE LineY, CenterPos


    FOR x = 1 TO LEN(TextToPrint$)
        _DELAY 0.11
        PRINT MID$(TextToPrint$, x, 1);
        CALL SoundBoard("Text", -1)
    NEXT x
END SUB




'*********************************************************
'* Draws the exterior walls in a zone.
'*
'*
'*********************************************************

SUB DrawWalls
    _DEST Collision&
    WallThickness = 90
    LINE (Settings.LeftLimit - WallThickness, Settings.TopLimit - WallThickness)-(Settings.RightLimit + WallThickness, Settings.BottomLimit + WallThickness), _RGB(20, 0, 20), BF
    LINE (Settings.LeftLimit, Settings.TopLimit)-(Settings.RightLimit, Settings.BottomLimit), _RGB(0, 0, 0), BF

    IF ActiveMap.Type <> 2 THEN
        FOR y = 1 TO ActiveMap.SpriteLimitY
            FOR x = 1 TO ActiveMap.SpriteLimitX
                IF SpriteMatrix(x, y) > Settings.SpriteDivider THEN
                    IF SpriteMatrix(x, y) > Settings.SpriteDivider + 18 AND SpriteMatrix(x, y) < Settings.SpriteDivider + 34 THEN GOTO ForExit
                    IF SpriteMatrix(x, y) = Settings.SpriteDivider + 65 THEN GOTO ForExit


                    SpriteWidth = (Settings.SquareWidth / 2) - 1
                    SpriteHeight = (Settings.SquareHeight / 2) - 1
                    SpriteX = (Settings.SquareWidth * (x - 1)) + Settings.LeftLimit + SpriteWidth
                    Spritey = (Settings.SquareHeight * (y - 1)) + Settings.TopLimit + SpriteHeight

                    IF SpriteMatrix(x, y) > Settings.SpriteDivider AND SpriteMatrix(x, y) < Settings.SpriteDivider + 4 THEN
                        LINE (SpriteX - SpriteWidth - 1, Spritey - SpriteHeight - 1)-(SpriteX + SpriteWidth + 1, Spritey + SpriteHeight + 1), _RGB(30, x, y), BF
                    ELSE
                        LINE (SpriteX - SpriteWidth - 1, Spritey - SpriteHeight - 1)-(SpriteX + SpriteWidth + 1, Spritey + SpriteHeight + 1), _RGB(20, 0, 20), BF
                    END IF
                END IF
                ForExit:
            NEXT x
        NEXT y

        IF ActiveMap.Type = 4 THEN
            CenterOfMap = ((Settings.RezY - Settings.TopLimit) / 2) + Settings.TopLimit
            LINE (0, Settings.TopLimit)-(Settings.RezX, CenterOfMap), _RGB(20, 0, 20), BF
            IF Settings.SoldItem = 0 THEN CALL DrawStore
        END IF

    ELSE
        FOR y = 1 TO ActiveMap.SpriteLimitY
            FOR x = 1 TO ActiveMap.SpriteLimitX
                IF SpriteMatrix(x, y) > Settings.SpriteDivider THEN
                    IF SpriteMatrix(x, y) > Settings.SpriteDivider + 18 AND SpriteMatrix(x, y) < Settings.SpriteDivider + 34 THEN GOTO ForExit
                    IF SpriteMatrix(x, y) = Settings.SpriteDivider + 65 THEN GOTO ForExit2


                    SpriteWidth = (Settings.SquareWidth / 2) - 1
                    SpriteHeight = (Settings.SquareHeight / 2) - 1
                    SpriteX = (Settings.SquareWidth * (x - 1)) + Settings.LeftLimit + SpriteWidth
                    Spritey = (Settings.SquareHeight * (y - 1)) + Settings.TopLimit + SpriteHeight

                    IF SpriteMatrix(x, y) > Settings.SpriteDivider AND SpriteMatrix(x, y) < Settings.SpriteDivider + 4 THEN
                        LINE (SpriteX - SpriteWidth - 1, Spritey - SpriteHeight - 1)-(SpriteX + SpriteWidth + 1, Spritey + SpriteHeight + 1), _RGB(20, 0, 20), BF
                    ELSEIF SpriteMatrix(x, y) = Settings.SpriteDivider + 4 THEN
                        LINE (SpriteX - SpriteWidth - 1, Spritey - SpriteHeight - 1)-(SpriteX + SpriteWidth + 1, Spritey + SpriteHeight + 1), _RGB(130, 0, 20), BF 'EDITME when ladder is implimented
                    END IF
                END IF
                ForExit2:
            NEXT x
        NEXT y
    END IF
END SUB




'*********************************************************
'* Kills a specific sprite at x location in the Sprites
'* sprite matrix
'*
'*********************************************************
SUB KillSprite (x)
    IF Settings.Debug > 1 AND Sprites(x).Active = 1 THEN CALL PrintConsole("World -> Removing sprite#:" + STR$(x) + ": " + Sprites(x).Name, 2)

    IF LEFT$(Sprites(x).Name, 1) <> "-" THEN Sprites(x).Name = "-" + Sprites(x).Name
    Sprites(x).Active = 0
    Sprites(x).XPos = 0
    Sprites(x).YPos = 0
    Sprites(x).OXPos = 0
    Sprites(x).OYPos = 0
    Sprites(x).MoveX = 0
    Sprites(x).MoveY = 0
    Sprites(x).Direction = 0
    Sprites(x).Speed = 0
    Sprites(x).Size = 0
    Sprites(x).AnimationDelay = 0
    Sprites(x).Attacking = 0
    Sprites(x).MaxDelay = 0
    Sprites(x).Frame = 0
    Sprites(x).Frame2 = 0
    Sprites(x).Counter = 0
    Sprites(x).MaxCount = 0
    Sprites(x).Stage = 0
    Sprites(x).Pause = 0
    Sprites(x).Health = 0
    Sprites(x).Hurt = 0
    Sprites(x).HurtMove = 0
    Sprites(x).Damage = 0
    Sprites(x).AttackPointer = 0
    Sprites(x).Segments = 0
    Sprites(x).HP = 0
    Sprites(x).SnakeNum = 0
    Sprites(x).SnakeSize = 0
    Sprites(x).SnakeDivider = 0
    Sprites(x).Stunned = 0
    Sprites(x).WalkTime = 0
    Sprites(x).WalkAdd = 0
    Sprites(x).PatraLoc = 0
    Sprites(x).RadX = 0
    Sprites(x).RadY = 0
    Sprites(x).Angle = 0
    Sprites(x).Flash = 0
    Sprites(x).MaxFlash = 0
    Sprites(x).Ambient = 0
    Sprites(x).Jumping = 0
    Sprites(x).Master = 0
    Sprites(x).Misc = 0
    Sprites(x).Flying = 0
    Sprites(x).LootCategory = 0
    Sprites(x).Boss = 0
    Sprites(x).HardCount = 0
    Sprites(x).MoveCheckBypass = 0
    IF Player1.LikeLocked = x THEN Player1.LikeLocked = 0
END SUB





'*********************************************************
'* Kills a specific sprite at x location in the Projectile
'* sprite matrix
'*
'*********************************************************

SUB KillProjectile (x)
    IF Settings.Debug > 2 AND Projectile(x).Active = 1 THEN CALL PrintConsole("World -> Removing sprite#:" + STR$(x) + ": " + Sprites(x).Name, 3)

    IF Player1.HaveBook = 1 AND Projectile(x).PlayerSpawned = 1 AND Projectile(x).Name = "Magic" THEN
        CALL SpawnSprite("Flame", Projectile(x).XPos, Projectile(x).YPos, 0, 0, 0, 1)
        IF Settings.BlackedOut = 1 THEN
            CALL TransitionMap(LTRIM$(RTRIM$(Settings.CurrentZone)) + Settings.MapExtension, 7)
            Settings.BlackedOut = 0
        END IF
    END IF


    IF LEFT$(Projectile(x).Name, 1) <> "-" THEN Projectile(x).Name = "-" + Projectile(x).Name
    Projectile(x).Active = 0
    Projectile(x).XPos = 0
    Projectile(x).YPos = 0
    Projectile(x).MoveX = 0
    Projectile(x).MoveY = 0
    Projectile(x).Direction = 0
    Projectile(x).Size = 0
    Projectile(x).AnimationDelay = 0
    Projectile(x).MaxDelay = 0
    Projectile(x).Frame = 0
    Projectile(x).Counter = 0
    Projectile(x).Stage = 0
    Projectile(x).MobControl = 0
    Projectile(x).ShieldUpgrade = 0
    Projectile(x).Damage = 0
    Projectile(x).Distance = 0
    IF Projectile(x).PlayerSpawned = 1 THEN
        Player1.LockSecondary = 0
        Projectile(x).PlayerSpawned = 0
    END IF
END SUB






'*********************************************************
'* Kills a specific sprite at x location in the Special
'* sprite matrix
'*
'*********************************************************
SUB KillSpecialSprite (x)
    IF LEFT$(Special(x).Name, 2) = "P-" THEN Settings.UpdateUI = 1

    Special(x).Active = 0
    Special(x).AnimationDelay = 0
    Special(x).Counter = 0
    Special(x).Direction = 0
    Special(x).Frame = 0
    Special(x).MaxCount = 0
    Special(x).MaxDelay = 0
    Special(x).MoveX = 0
    Special(x).MoveY = 0
    Special(x).Name = "-" + Special(x).Name
    Special(x).Size = 0
    Special(x).Speed = 0
    Special(x).Stage = 0
    Special(x).XPos = 0
    Special(x).YPos = 0
END SUB



'*********************************************************
'* Kills all active sprites in the Special matrix
'*
'*
'*********************************************************
SUB KillAllSpecialSprites
    FOR x = 1 TO Settings.MatrixSize
        IF Special(x).Active = 1 THEN CALL KillSpecialSprite(x)
    NEXT x
END SUB



'*********************************************************
'* Kills all active sprites in the Sprite matrix
'*
'*
'*********************************************************
SUB KillAllRegSprites
    FOR x = 1 TO Settings.MatrixSize
        IF Sprites(x).Active = 1 THEN CALL KillSprite(x)
    NEXT x
END SUB




'*********************************************************
'* Kills all active sprites in every matrix
'*
'*
'*********************************************************
SUB KillAllSprites
    FOR x = 1 TO Settings.MatrixSize
        IF Sprites(x).Active = 1 THEN CALL KillSprite(x)
        IF Projectile(x).Active = 1 THEN CALL KillProjectile(x)
        IF Special(x).Active = 1 THEN CALL KillSpecialSprite(x)
    NEXT x

    FOR x = 1 TO 4
        Snakes(x) = 0
    NEXT x

    Attack(2).Active = 0
    Attack(3).Active = 0
    _DEST Sprites&
    CLS
    _CLEARCOLOR _RGB(0, 0, 0)
END SUB



'*********************************************************
'* MasterProcs were intended to kill all mobs when a specific mob
'* was killed. These work, but are not used in the game
'*
'*********************************************************

SUB MasterProc (x)
    FOR y = 1 TO Settings.MatrixSize
        IF Sprites(x).Name = Sprites(y).Name THEN
            'spawn death sprite at y location
            CALL KillSprite(y)
        END IF
    NEXT y
END SUB

SUB MasterProc2 (x)
    FOR y = 1 TO Settings.MatrixSize
        IF Sprites(y).Master = x THEN
            'spawn death sprite at y location
            CALL KillSprite(y)
        END IF
    NEXT y
END SUB




'*********************************************************
'* Checks what item was colected based on its 'SpriteNum' location
'* in the 'Sprites' matrix. Will kill sprite and change the needed stats
'*
'*********************************************************
SUB ItemCollected (SpriteNum)
    IF Sprites(SpriteNum).Name = "Heart" THEN
        CALL SoundBoard("Heart", -1)
        CALL KillSprite(SpriteNum)
        Player1.Health = Player1.Health + 2
        IF Player1.Health > Player1.MaxHealth THEN Player1.Health = Player1.MaxHealth

    ELSEIF Sprites(SpriteNum).Name = "Rupee" THEN
        CALL SoundBoard("Rupee", -1)
        CALL KillSprite(SpriteNum)
        Settings.ModifyCash = Settings.ModifyCash + 1

    ELSEIF Sprites(SpriteNum).Name = "5Rupe" THEN
        CALL SoundBoard("Rupee", -1)
        CALL KillSprite(SpriteNum)
        Settings.ModifyCash = Settings.ModifyCash + 5

    ELSEIF Sprites(SpriteNum).Name = "Fairy" THEN
        CALL SoundBoard("Item", -1)
        CALL KillSprite(SpriteNum)
        Player1.Health = Player1.MaxHealth

    ELSEIF Sprites(SpriteNum).Name = "Clock" THEN
        CALL SoundBoard("Item", -1)
        CALL KillSprite(SpriteNum)
        CALL StunEveryone
        Player1.Clock = 1

    ELSEIF Sprites(SpriteNum).Name = "P-Bmb" THEN
        IF Player1.Bombs = -1 THEN Player1.Bombs = 0
        CALL SoundBoard("Item", -1)
        CALL KillSprite(SpriteNum)
        Player1.Bombs = Player1.Bombs + 1

    ELSEIF Sprites(SpriteNum).Name = "P-Bm4" THEN
        IF Player1.Bombs = -1 THEN Player1.Bombs = 0
        CALL SoundBoard("Item", -1)
        CALL KillSprite(SpriteNum)
        Player1.Bombs = Player1.Bombs + 4

    END IF

    Settings.UpdateUI = 1
END SUB




'*********************************************************
'* Use when you need to remove the directory and file extention
'* from a zone
'*
'*********************************************************
FUNCTION CleanZoneTarget$ (Target$)
    Target$ = LTRIM$(RTRIM$(Target$))

    IF LEFT$(Target$, 6) = "World\" THEN Target$ = MID$(Target$, 7, LEN(Target$))
    IF RIGHT$(Target$, 4) = ".msz" THEN Target$ = LEFT$(Target$, LEN(Target$) - 4)

    CleanZoneTarget$ = Target$
END FUNCTION




'*********************************************************
'* Checks to see if zone has been previously cleared.
'* If so spawn matrix is ignored
'*
'*********************************************************

FUNCTION AlreadyOnClearList
    AlreadyOnClearList = 0
    FOR x = 1 TO Settings.ClearedArraySize
        IF ClearedArray(x) = Settings.CurrentZone THEN AlreadyOnClearList = 1
    NEXT x
END FUNCTION

'*********************************************************
'* Clears the entire cleared zone list
'*
'*
'*********************************************************

SUB ClearZoneList
    FOR C = 1 TO Settings.ClearedArraySize
        ClearedArray(C) = ""
    NEXT C
    IF Settings.Debug <> 0 THEN CALL PrintConsole("Engine -> Reset cleared zone list", 1)
END SUB





'*********************************************************
'* Used to face link properly when zoning to underground
'*
'*
'*********************************************************

SUB FaceLinkProper
    IF Player1.XPos < (Settings.RezX / 2) - 150 THEN
        Player1.Direction = 2
    ELSEIF Player1.XPos > (Settings.RezX / 2) + 150 THEN
        Player1.Direction = 4
    ELSEIF Player1.YPos > Settings.RezY - 200 THEN
        Player1.Direction = 1
    ELSE
        Player1.Direction = 3
    END IF
END SUB






'*********************************************************
'* Handles all the transitioning required to zone. Also fade out and in
'*
'*
'*********************************************************

SUB TransitionMap (MapFile$, TransitionType)
    IF Settings.Debug <> 0 THEN CALL PrintConsole("Engine -> Transition#:" + STR$(TransitionType) + " selected.", 1)

    IF TransitionType < 6 OR TransitionType > 8 THEN
        SlideSpeed = 5
        _PUTIMAGE , Foreground&, Scene&
        _DEST Sprites&
        CLS
        _CLEARCOLOR _RGB(0, 0, 0)

        IF TransitionType = 5 OR TransitionType > 8 THEN CALL LoadMap(MapFile$, 0)

        CALL DrawMap(Transition&, Transition&, 1, 1)
        _DEST Transition&
        _CLEARCOLOR _RGB(0, 26, 255)
        _DEST Buffer&

        Adjustment = 200
        DrawLink = 0
        IF ActiveMap.Type = 1 THEN
            Adjustment = 215
            DrawLink = 1
        END IF
    END IF

    IF TransMap.BGM = 4 THEN TransMap.BGM = 10

    IF TransitionType = 1 THEN
        FOR YOffset = 1 TO Settings.RezY - Adjustment STEP SlideSpeed
            CALL CheckForAudioLoop
            _PUTIMAGE , WaterScreen&, Buffer&
            _PUTIMAGE (0, YOffset - (Settings.RezY - Adjustment)), Transition&, Buffer&
            _PUTIMAGE (0, YOffset), Scene&, Buffer&

            LINE (0, 0)-(Settings.RezX, 200), _RGB(0, 0, 0), BF
            _PUTIMAGE , UI&, Buffer&
            Player1.YPos = Player1.YPos + SlideSpeed
            IF Player1.YPos > Settings.RezY - (Player1.Size + 25) THEN Player1.YPos = Settings.RezY - (Player1.Size + 25)
            IF DrawLink = 1 THEN CALL DrawPlayerSprite(Buffer&)
            _PUTIMAGE , Buffer&, Display&
        NEXT YOffset

    ELSEIF TransitionType = 2 THEN
        FOR XOffset = Settings.RezX TO 1 STEP SlideSpeed * -1
            CALL CheckForAudioLoop
            _PUTIMAGE , WaterScreen&, Buffer&
            _PUTIMAGE (XOffset - (Adjustment - 200), 0), Transition&, Buffer&
            _PUTIMAGE (0 - (Settings.RezX - XOffset), 0), Scene&, Buffer&

            LINE (0, 0)-(Settings.RezX, 200), _RGB(0, 0, 0), BF
            _PUTIMAGE , UI&, Buffer&
            Player1.XPos = Player1.XPos - SlideSpeed
            IF Player1.XPos < Player1.Size + 25 THEN Player1.XPos = Player1.Size + 25
            IF DrawLink = 1 THEN CALL DrawPlayerSprite(Buffer&)
            _PUTIMAGE , Buffer&, Display&
        NEXT XOffset

    ELSEIF TransitionType = 3 THEN
        FOR YOffset = Settings.RezY TO Adjustment STEP SlideSpeed * -1
            CALL CheckForAudioLoop
            _DEST WaterScreen&
            LINE (0, 0)-(Settings.RezX, 200), _RGB(0, 26, 255), BF
            _DEST Buffer&
            _PUTIMAGE , WaterScreen&, Buffer&
            _PUTIMAGE (0, YOffset - Adjustment), Transition&, Buffer&
            _PUTIMAGE (0, 0 - (Settings.RezY - YOffset)), Scene&, Buffer&

            LINE (0, 0)-(Settings.RezX, 200), _RGB(0, 0, 0), BF
            _PUTIMAGE , UI&, Buffer&
            Player1.YPos = Player1.YPos - SlideSpeed
            IF Player1.YPos < 225 + Player1.Size THEN Player1.YPos = Player1.Size + 225
            IF DrawLink = 1 THEN CALL DrawPlayerSprite(Buffer&)
            _PUTIMAGE , Buffer&, Display&
        NEXT YOffset

    ELSEIF TransitionType = 4 THEN
        FOR XOffset = 0 TO Settings.RezX STEP SlideSpeed
            CALL CheckForAudioLoop
            _PUTIMAGE , WaterScreen&, Buffer&
            _PUTIMAGE (XOffset - Settings.RezX + (Adjustment - 200), 0), Transition&, Buffer&
            _PUTIMAGE (0 + XOffset, 0), Scene&, Buffer&

            LINE (0, 0)-(Settings.RezX, 200), _RGB(0, 0, 0), BF
            _PUTIMAGE , UI&, Buffer&
            Player1.XPos = Player1.XPos + SlideSpeed
            IF Player1.XPos > Settings.RezX - (Player1.Size + 25) THEN Player1.XPos = Settings.RezX - (Player1.Size + 25)
            IF DrawLink = 1 THEN CALL DrawPlayerSprite(Buffer&)
            _PUTIMAGE , Buffer&, Display&
        NEXT XOffset

    ELSEIF TransitionType = 5 THEN

        IF ActiveMap.Type = 1 THEN
            CALL JukeBox(-3)
            Player1.PrevX = Player1.XPos
            Player1.PrevY = Player1.YPos
            _DEST Scene&
            _CLEARCOLOR _RGB(2, 2, 2)
            CALL SoundBoard("Stairs", -1)

            FOR XOffset = 0 TO Settings.RezX / 2 STEP 2
                _DEST Sprites&
                CLS
                _CLEARCOLOR _RGB(0, 0, 0)
                LINE (Player1.XPos - 100, Player1.YPos - 100)-(Player1.XPos + 100, Player1.YPos + 100), _RGB(1, 1, 1), BF
                Player1.YPos = Player1.YPos + .1
                IF XOffset MOD 4 = 0 THEN CALL DrawPlayer
                CALL DrawPlayerSprite(Sprites&)

                _DEST Buffer&
                _PUTIMAGE , WaterScreen&, Buffer&
                _PUTIMAGE , Sprites&, Buffer&
                _PUTIMAGE , Scene&, Buffer&
                _PUTIMAGE , UI&, Buffer&

                LINE (0, 199)-(XOffset, Settings.RezY), _RGB(1, 1, 1), BF
                LINE (Settings.RezX, 199)-(Settings.RezX - XOffset, Settings.RezY), _RGB(1, 1, 1), BF

                _PUTIMAGE , Buffer&, Display&
            NEXT XOffset

            CALL PositionPlayer(ActiveMap.Type)
            CALL FaceLinkProper

            FOR XOffset = Settings.RezX / 2 TO 0 STEP -1
                _PUTIMAGE , Transition&, Buffer&
                _DEST Buffer&
                LINE (0, 199)-(XOffset, Settings.RezY), _RGB(1, 1, 1), BF
                LINE (Settings.RezX, 199)-(Settings.RezX - XOffset, Settings.RezY), _RGB(1, 1, 1), BF
                _PUTIMAGE , Buffer&, Display&
            NEXT XOffset
            CALL JukeBox(TransMap.BGM)


        ELSE
            CALL JukeBox(-3)

            _DEST Scene&

            FOR XOffset = 0 TO Settings.RezX / 2 STEP 1
                LINE (0, 199)-(XOffset, Settings.RezY), _RGB(1, 1, 1), BF
                LINE (Settings.RezX, 199)-(Settings.RezX - XOffset, Settings.RezY), _RGB(1, 1, 1), BF
                _PUTIMAGE , Scene&, Display&
            NEXT XOffset

            _DEST Transition&
            _CLEARCOLOR _RGB(2, 2, 2)
            Player1.Direction = 3

            IF ActiveMap.Type > 2 THEN
                IF ActiveMap.Type = 3 AND ActiveMap.SubType = 2 THEN
                    CALL PositionPlayer(ActiveMap.Type)
                    CALL CenterLink
                    Player1.YPos = Player1.YPos + 60
                ELSE
                    Player1.XPos = Player1.PrevX
                    Player1.YPos = Player1.PrevY + 20
                END IF
            ELSE
                CALL PositionPlayer(ActiveMap.Type)
                CALL CenterLink
                Player1.YPos = Player1.YPos + 60
            END IF

            CALL SoundBoard("Stairs", -1)

            FOR XOffset = Settings.RezX / 2 TO 0 STEP -2
                _DEST Sprites&
                CLS
                _CLEARCOLOR _RGB(0, 0, 0)
                LINE (Player1.XPos - 100, Player1.YPos - 100)-(Player1.XPos + 100, Player1.YPos + 100), _RGB(1, 1, 1), BF
                Player1.YPos = Player1.YPos - .1
                IF XOffset MOD 4 = 0 THEN CALL DrawPlayer
                CALL DrawPlayerSprite(Sprites&)

                _DEST Buffer&
                _PUTIMAGE , WaterScreen&, Buffer&
                _PUTIMAGE , Sprites&, Buffer&
                _PUTIMAGE , Transition&, Buffer&
                _PUTIMAGE , UI&, Buffer&

                _DEST Buffer&
                LINE (0, 199)-(XOffset, Settings.RezY), _RGB(1, 1, 1), BF
                LINE (Settings.RezX, 199)-(Settings.RezX - XOffset, Settings.RezY), _RGB(1, 1, 1), BF

                _PUTIMAGE , Buffer&, Display&
            NEXT XOffset
            CALL JukeBox(TransMap.BGM)
        END IF

    ELSEIF TransitionType = 6 THEN
        FOR x = 0 TO 250 STEP 5
            Settings.Alpha = x
            Settings.UpdateUI = 1
            CALL AssembleDisplay
            CALL CheckForAudioLoop
        NEXT x

    ELSEIF TransitionType = 7 THEN
        FOR x = 250 TO 0 STEP -5
            Settings.Alpha = x
            Settings.UpdateUI = 1
            CALL AssembleDisplay
            CALL CheckForAudioLoop
        NEXT x

    ELSEIF TransitionType = 8 THEN
        FOR x = 0 TO 250 STEP 50
            Settings.Alpha = x
            Settings.UpdateUI = 1
            CALL AssembleDisplay
            CALL CheckForAudioLoop
        NEXT x

        CALL LoadMap(MapFile$, 0)
        CALL SetStartPos(1, 0)
        CALL DrawMap(Scene&, Foreground&, 1, 0)
        IF TransMap.BGM = 4 THEN TransMap.BGM = 10
        CALL JukeBox(TransMap.BGM)

        FOR x = 250 TO 0 STEP -50
            Settings.Alpha = x
            Settings.UpdateUI = 1
            CALL AssembleDisplay
            CALL CheckForAudioLoop
        NEXT x

    ELSEIF TransitionType = 9 THEN
        CALL JukeBox(-3)

        _DEST Scene&
        _CLEARCOLOR _RGB(2, 2, 2)
        CALL SoundBoard("Stairs", -1)

        FOR XOffset = 0 TO Settings.RezX / 2 STEP 2
            _DEST Sprites&
            CLS
            _CLEARCOLOR _RGB(0, 0, 0)
            CALL DrawPlayerSprite(Sprites&)

            _DEST Buffer&
            _PUTIMAGE , WaterScreen&, Buffer&
            _PUTIMAGE , Sprites&, Buffer&
            _PUTIMAGE , Scene&, Buffer&
            _PUTIMAGE , UI&, Buffer&

            LINE (0, 199)-(XOffset, Settings.RezY), _RGB(1, 1, 1), BF
            LINE (Settings.RezX, 199)-(Settings.RezX - XOffset, Settings.RezY), _RGB(1, 1, 1), BF


            _PUTIMAGE , Buffer&, Display&
        NEXT XOffset

        CALL PositionPlayer(ActiveMap.Type)
        CALL FaceLinkProper

        FOR XOffset = Settings.RezX / 2 TO 0 STEP -1
            _PUTIMAGE , Transition&, Buffer&
            _DEST Buffer&
            LINE (0, 199)-(XOffset, Settings.RezY), _RGB(1, 1, 1), BF
            LINE (Settings.RezX, 199)-(Settings.RezX - XOffset, Settings.RezY), _RGB(1, 1, 1), BF
            _PUTIMAGE , Buffer&, Display&
        NEXT XOffset
        CALL JukeBox(TransMap.BGM)

    ELSEIF TransitionType = 10 THEN
        CALL JukeBox(-3)
        _DEST Scene&
        _CLEARCOLOR _RGB(2, 2, 2)

        CALL PositionPlayer(ActiveMap.Type)

        FOR XOffset = Settings.RezX / 2 TO 0 STEP -1
            _PUTIMAGE , Transition&, Buffer&
            _DEST Buffer&
            LINE (0, 199)-(XOffset, Settings.RezY), _RGB(1, 1, 1), BF
            LINE (Settings.RezX, 199)-(Settings.RezX - XOffset, Settings.RezY), _RGB(1, 1, 1), BF
            _PUTIMAGE , Buffer&, Display&
        NEXT XOffset
        CALL JukeBox(TransMap.BGM)
    END IF

    ActiveMap = TransMap
    ActiveFile$ = MapFile$ ' World\"+ MapFile$
    CALL SetMapSize
    FOR y = 1 TO ActiveMap.SpriteLimitY
        FOR x = 1 TO ActiveMap.SpriteLimitX
            SpriteMatrix(x, y) = TransMatrix(x, y)
        NEXT x
    NEXT y
    CALL DrawMap(Scene&, Foreground&, 0, 0)

    _DEST Collision&
    CLS
    _DEST Sprites&
    CLS
    _CLEARCOLOR _RGB(0, 0, 0)

    IF Settings.Debug <> 0 THEN
        _DEST DebugScreen&
        CLS
        _CLEARCOLOR _RGB(0, 0, 0)

    END IF

    CALL SoundBoard("0", ActiveMap.AmbientSound)
    Settings.UpdateUI = 1
END SUB









'*********************************************************
'* Used by TransitionMap to figure out how to relocate the player
'*
'*
'*********************************************************

SUB PositionPlayer (CrossoverCheck)
    IF CrossoverCheck = 1 AND TransMap.Type = 2 THEN
        CALL SetStartPos(1, 0)
        Player1.Direction = 1
    ELSEIF CrossoverCheck = 2 AND TransMap.Type = 1 THEN
        CALL SetStartPos(31 + Settings.SpriteDivider, 0)
        Player1.Direction = 3
    ELSEIF CrossoverCheck = 1 AND TransMap.Type = 3 THEN
        CALL SetStartPos(1, 0)
        Player1.Direction = 1
    ELSEIF CrossoverCheck = 3 AND TransMap.Type = 1 THEN
        CALL SetStartPos(31 + Settings.SpriteDivider, 0)
        Player1.Direction = 3
    ELSEIF CrossoverCheck > 2 AND TransMap.Type = 1 THEN
        CALL DrawMap(Scene&, Foreground&, 0, 0)
    ELSEIF CrossoverCheck = 1 AND TransMap.Type > 2 THEN
        Player1.Direction = 1
        Player1.XPos = Settings.RezX / 2
        Player1.YPos = 700
    END IF
END SUB
