'************************************************************
'* Enviroment.bi                         6/13/2021
'*
'* Contains the functions and subs that have to deal with altering 
'* or animating the world
'*
'* Only used by game
'************************************************************


'************************************************************
'* SUB AnimateWater
'*
'* Handles animating the water, not waterfalls
'************************************************************
SUB AnimateWater
    Water.Counter = Water.Counter + 1
    IF Water.Counter = Water.MaxCount THEN
        Water.Counter = 0
        Water.Frame = Water.Frame - 1
        IF Water.Frame < 1 THEN Water.Frame = 4
        IF ActiveMap.Type = 1 THEN
            _DEST WaterScreen&
            CLS
            _PUTIMAGE (0, 0)-(Settings.RezX, Settings.RezY), WaterSprites(Water.Frame), WaterScreen&
        END IF
    END IF
END SUB


'************************************************************
'* SUB UnlockDoor
'*
'* Controls the doors that need a key.
'* Check1& and Check2& are the two points in front of Link 
'* That are used to detect his collisions.
'************************************************************
SUB UnlockDoor (Check1&, Check2&)
    IF Settings.Debug <> 0 THEN CALL PrintConsole("Enviroment -> Unlocking Door", 1)

    Settings.ErrorTracker = STR$(_BLUE(Check1&))
    Player1.Keys = Player1.Keys - 1
    IF _BLUE(Check1&) = 1 OR _BLUE(Check2&) = 1 THEN
        ActiveMap.DoorN = 1
        CALL LoadLinkMap(RTRIM$(ActiveMap.NFile), 1)
        IF LinkMap.DoorS = 2 AND LinkMap.Type = ActiveMap.Type THEN CALL LinkMaps(RTRIM$(ActiveMap.NFile), 1, 1)
    ELSEIF _BLUE(Check1&) = 2 OR _BLUE(Check2&) = 2 THEN
        ActiveMap.DoorE = 1
        CALL LoadLinkMap(RTRIM$(ActiveMap.EFile), 2)
        IF LinkMap.DoorW = 2 AND LinkMap.Type = ActiveMap.Type THEN CALL LinkMaps(RTRIM$(ActiveMap.EFile), 2, 1)
    ELSEIF _BLUE(Check1&) = 3 OR _BLUE(Check2&) = 3 THEN
        ActiveMap.DoorS = 1
        CALL LoadLinkMap(RTRIM$(ActiveMap.SFile), 3)
        IF LinkMap.DoorN = 2 AND LinkMap.Type = ActiveMap.Type THEN CALL LinkMaps(RTRIM$(ActiveMap.SFile), 3, 1)
    ELSEIF _BLUE(Check1&) = 4 OR _BLUE(Check2&) = 4 THEN
        ActiveMap.DoorW = 1
        CALL LoadLinkMap(RTRIM$(ActiveMap.WFile), 4)
        IF LinkMap.DoorE = 2 AND LinkMap.Type = ActiveMap.Type THEN CALL LinkMaps(RTRIM$(ActiveMap.EFile), 4, 1)
    END IF
    CALL SaveMap
    CALL SaveGame("UnlockDoor")
    CALL DrawMap(Scene&, Foreground&, 0, 0)
    CALL SoundBoard("Door", -1)
    CALL GetPlayerCollisionData(Check1&, Check2&)
END SUB



'************************************************************
'* SUB CheckLockdown
'*
'* Controls the doors in the dungeon room. 
'************************************************************
SUB CheckLockdown (ForceOpen)
    IF ActiveMap.OpenCondition > 0 AND AreAllDead = 0 AND Player1.InRoom = 1 AND Settings.LockDown = 0 THEN
        CALL SoundBoard("Door", -1)
        Settings.LockDown = 1
        CALL DrawMap(Scene&, Foreground&, 0, 0)
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Enviroment -> Detected Link inside room. Locking room down.", 1)
    ELSEIF ActiveMap.OpenCondition = 2 AND Player1.InRoom = 1 AND Settings.LockDown = 0 THEN
        CALL SoundBoard("Door", -1)
        Settings.LockDown = 1
        CALL DrawMap(Scene&, Foreground&, 0, 0)
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Enviroment -> Detected Link inside room. Locking room down.", 1)
    ELSEIF Settings.LockDown = 1 AND AreAllDead = 1 AND ActiveMap.OpenCondition = 1 THEN
        CALL SoundBoard("Door", -1)
        Settings.LockDown = -1
        CALL DrawMap(Scene&, Foreground&, 0, 0)
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Enviroment -> Detected cleared room. Opening doors.", 1)
    ELSEIF ForceOpen = 1 AND Settings.LockDown = 1 THEN
        CALL SoundBoard("Door", -1)
        Settings.LockDown = -1
        Settings.ShowStairs = 1
        CALL DrawMap(Scene&, Foreground&, 0, 0)
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Enviroment -> Force open triggered. Opening doors.", 1)
    ELSEIF ForceOpen = 2 AND Settings.LockDown = -1 THEN
        CALL SoundBoard("Door", -1)
        Settings.LockDown = 1
        CALL DrawMap(Scene&, Foreground&, 0, 0)
        IF Settings.Debug <> 0 THEN CALL PrintConsole("Enviroment -> Force close triggered. Closing doors.", 1)
    END IF
END SUB



'************************************************************
'* SUB BombWall
'*
'* Called when a bomb was detected to be close enough to open
'* a hole if the wall is set to be bombable.
'* It will alter the map, save to file, save game.
'************************************************************
SUB BombWall (Wall, LinkFile$)
    IF Settings.Debug <> 0 THEN CALL PrintConsole("Enviroment -> Detected explosion at bombable wall.", 1)

    IF Wall = 1 THEN
        ActiveMap.DoorN = 5
    ELSEIF Wall = 2 THEN
        ActiveMap.DoorE = 5
    ELSEIF Wall = 3 THEN
        ActiveMap.DoorS = 5
    ELSE
        ActiveMap.DoorW = 5
    END IF

    CALL LoadLinkMap(RTRIM$(LinkFile$), Wall)
    CALL LinkMaps(RTRIM$(LinkFile$), Wall, 5)
    CALL SaveMap
    CALL SaveGame("BombWall")
    CALL DrawMap(Scene&, Foreground&, 0, 0)
    CALL SoundBoard("Secret", -1)
    CALL GetPlayerCollisionData(Check1&, Check2&)
END SUB