'*********************************************************
'* Global.bi                                       6/12/21
'*
'* Contains all the code the world editor and main game share
'*********************************************************



'*********************************************************
'* RepeatTile
'*
'* Will repeat a specific tile across the entire map, used to save
'* space on the object matrix for other things. Dungeons Only!
'*
'* TileNum - The number assigned to the tile you want to repeat
'* Pallet - Used for dungeons 1 - 9
'* XLimit and YLimit - Size of object matrix
'* Target$ - Screen to output to
'*********************************************************
SUB RepeatTile (TileNum, Pallet, XLimit, YLimit, Target&)
    SpriteWidth = (Settings.SquareWidth / 2) - 1
    SpriteHeight = (Settings.SquareHeight / 2) - 1

    FOR y = 1 TO YLimit
        FOR x = 1 TO XLimit
            SpriteX = (Settings.SquareWidth * (x - 1)) + Settings.LeftLimit + SpriteWidth
            SpriteY = (Settings.SquareHeight * (y - 1)) + Settings.TopLimit + SpriteHeight

            _PUTIMAGE (SpriteX - SpriteWidth - 1, SpriteY - SpriteHeight - 1)-(SpriteX + SpriteWidth + 1, SpriteY + SpriteHeight + 1), DungTile(TileNum - Settings.SpriteDivider, Pallet), Target&
        NEXT x
    NEXT y
END SUB




'*********************************************************
'* LoadOverworldTiles
'*
'* Used to load the artwork used in overworld
'*********************************************************
SUB LoadOverworldTiles
    IF Settings.Debug <> 0 THEN
        CALL PrintConsole("Engine -> Loading Overworld tile set", 1)
    ELSE
        PRINT "Engine -> Loading Overworld tile set"
    END IF

    OverworldTile(1) = _LOADIMAGE("Images\Armos.png")
    OverworldTile(2) = OverworldTile(1)
    OverworldTile(3) = OverworldTile(2)
    OverworldTile(2) = SwapColor(OverworldTile(2), _RGB(192, 56, 0), _RGB(0, 156, 0))
    OverworldTile(3) = SwapColor(OverworldTile(3), _RGB(192, 56, 0), _RGB(255, 255, 255))

    OverworldTile(4) = _LOADIMAGE("Images\BTree1.png")
    OverworldTile(5) = _LOADIMAGE("Images\BTree2.png")
    OverworldTile(6) = _LOADIMAGE("Images\BTree3.png")
    OverworldTile(7) = _LOADIMAGE("Images\BTree4.png")
    OverworldTile(8) = _LOADIMAGE("Images\BTree5.png")
    y = 4
    Col~& = _RGB(0, 156, 0)
    FOR x = 9 TO 18
        OverworldTile(x) = OverworldTile(y)
        OverworldTile(x) = SwapColor(OverworldTile(x), _RGB(192, 56, 0), Col~&)
        y = y + 1
        IF y = 9 THEN
            y = 4
            Col~& = _RGB(255, 255, 255)
        END IF
    NEXT x

    OverworldTile(19) = _LOADIMAGE("Images\Dirt.png")
    OverworldTile(20) = OverworldTile(19)
    OverworldTile(20) = SwapColor(OverworldTile(20), _RGB(255, 192, 192), _RGB(122, 122, 122))

    OverworldTile(21) = _LOADIMAGE("Images\Dock.png")

    OverworldTile(22) = _LOADIMAGE("Images\Dry1.png")
    OverworldTile(23) = _LOADIMAGE("Images\Dry2.png")
    OverworldTile(24) = _LOADIMAGE("Images\Dry3.png")
    OverworldTile(25) = _LOADIMAGE("Images\Dry4.png")
    OverworldTile(26) = _LOADIMAGE("Images\Dry5.png")
    OverworldTile(27) = _LOADIMAGE("Images\Dry6.png")
    OverworldTile(28) = _LOADIMAGE("Images\Dry7.png")
    OverworldTile(29) = _LOADIMAGE("Images\Dry8.png")
    OverworldTile(30) = _LOADIMAGE("Images\Dry9.png")

    OverworldTile(31) = _LOADIMAGE("Images\Hole.png")

    OverworldTile(32) = _LOADIMAGE("Images\Ladder.png")
    OverworldTile(33) = OverworldTile(32)
    OverworldTile(33) = SwapColor(OverworldTile(33), _RGB(192, 56, 0), _RGB(255, 255, 255))

    OverworldTile(34) = _LOADIMAGE("Images\Rock1.png")
    OverworldTile(35) = _LOADIMAGE("Images\Rock2.png")
    OverworldTile(36) = _LOADIMAGE("Images\Rock3.png")
    OverworldTile(37) = _LOADIMAGE("Images\Rock4.png")
    OverworldTile(38) = _LOADIMAGE("Images\Rock5.png")
    OverworldTile(39) = _LOADIMAGE("Images\Rock6.png")
    y = 34
    Col~& = _RGB(0, 156, 0)
    FOR x = 40 TO 51
        OverworldTile(x) = OverworldTile(y)
        OverworldTile(x) = SwapColor(OverworldTile(x), _RGB(192, 56, 0), Col~&)
        y = y + 1
        IF y = 40 THEN
            y = 34
            Col~& = _RGB(255, 255, 255)
        END IF
    NEXT x

    OverworldTile(52) = _LOADIMAGE("Images\Shore1.png")
    OverworldTile(53) = _LOADIMAGE("Images\Shore2.png")
    OverworldTile(54) = _LOADIMAGE("Images\Shore3.png")
    OverworldTile(55) = _LOADIMAGE("Images\Shore4.png")
    OverworldTile(56) = _LOADIMAGE("Images\Shore5.png")
    OverworldTile(57) = _LOADIMAGE("Images\Shore6.png")
    OverworldTile(58) = _LOADIMAGE("Images\Shore7.png")
    OverworldTile(59) = _LOADIMAGE("Images\Shore8.png")
    OverworldTile(60) = _LOADIMAGE("Images\Shore9.png")
    OverworldTile(61) = _LOADIMAGE("Images\Shore10.png")
    OverworldTile(62) = _LOADIMAGE("Images\Shore11.png")
    OverworldTile(63) = _LOADIMAGE("Images\Shore12.png")
    OverworldTile(64) = _LOADIMAGE("Images\Shore13.png")

    OverworldTile(65) = _LOADIMAGE("Images\Stairs.png")

    OverworldTile(66) = _LOADIMAGE("Images\Temple1.png")
    OverworldTile(67) = _LOADIMAGE("Images\Temple2.png")
    OverworldTile(68) = _LOADIMAGE("Images\Temple3.png")
    OverworldTile(69) = _LOADIMAGE("Images\Temple4.png")
    OverworldTile(70) = _LOADIMAGE("Images\Temple5.png")
    OverworldTile(71) = _LOADIMAGE("Images\Temple6.png")
    y = 66
    Col~& = _RGB(0, 156, 0)
    FOR x = 72 TO 83
        OverworldTile(x) = OverworldTile(y)
        OverworldTile(x) = SwapColor(OverworldTile(x), _RGB(192, 56, 0), Col~&)
        y = y + 1
        IF y = 72 THEN
            y = 66
            Col~& = _RGB(255, 255, 255)
        END IF
    NEXT x

    OverworldTile(84) = _LOADIMAGE("Images\Tombstone.png")

    OverworldTile(85) = _LOADIMAGE("Images\Tree.png")
    OverworldTile(86) = OverworldTile(85)
    OverworldTile(86) = SwapColor(OverworldTile(86), _RGB(0, 156, 0), _RGB(192, 56, 0))

    OverworldTile(87) = _LOADIMAGE("Images\Turtle.png")
    OverworldTile(88) = OverworldTile(87)
    OverworldTile(88) = SwapColor(OverworldTile(88), _RGB(0, 156, 0), _RGB(192, 56, 0))
    OverworldTile(89) = OverworldTile(87)
    OverworldTile(89) = SwapColor(OverworldTile(89), _RGB(0, 156, 0), _RGB(255, 255, 255))

    OverworldTile(90) = OverworldTile(38)
    OverworldTile(91) = OverworldTile(44)
    OverworldTile(92) = OverworldTile(50)
    OverworldTile(93) = OverworldTile(85)
    OverworldTile(94) = OverworldTile(86)
    OverworldTile(95) = _LOADIMAGE("Images\Cave.png")

    IF Settings.EditorActive = 1 THEN
        OverworldTile(96) = _LOADIMAGE("Images\Raft1.png")

        OverworldTile(97) = OverworldTile(22)
        OverworldTile(98) = OverworldTile(23)
        OverworldTile(99) = OverworldTile(24)
        OverworldTile(100) = OverworldTile(25)
        OverworldTile(101) = OverworldTile(26)
        OverworldTile(102) = OverworldTile(27)
        OverworldTile(103) = OverworldTile(28)
        OverworldTile(104) = OverworldTile(29)
        OverworldTile(105) = OverworldTile(30)
        OverworldTile(106) = OverworldTile(65)
    ELSE
        OverworldTile(96) = OverworldTile(56)
    END IF
END SUB





'*******************************************************************
'* Function SwapColor
'* Contributed by Petr on QB64
'* Swaps colors on an image already loaded
'*
'* handle - image to be processed
'* oldcolor~& - RGB of color you want to replace
'* newcolor~& - RGB of color you want to change to
'*******************************************************************
FUNCTION SwapColor (handle&, oldcolor~&, newcolor~&)
    DIM m AS _MEM, c AS _UNSIGNED LONG
    SwapColor = _COPYIMAGE(handle&, 32)
    m = _MEMIMAGE(SwapColor)
    DO UNTIL a& = m.SIZE '- 4
        c~& = _MEMGET(m, m.OFFSET + a&, _UNSIGNED LONG)
        IF c~& = oldcolor~& THEN _MEMPUT m, m.OFFSET + a&, newcolor~&
        a& = a& + 4
    LOOP
    _MEMFREE m
END FUNCTION


'*******************************************************************
'* Sub PalletteSwap
'* Swaps pallet of dungeon tiles to specified dungeon pallette from original game
'*
'* Handle& - image to be processed
'* LevNum - dungeon pallette to swap to
'*******************************************************************
SUB PalletteSwap (LevNum, Handle&)

    IF LevNum = 2 THEN
        Handle& = SwapColor(Handle&, _RGB(0, 132, 132), _RGB(66, 0, 255)) '     main color
        Handle& = SwapColor(Handle&, _RGB(0, 255, 255), _RGB(66, 132, 255)) '   highlight color
        Handle& = SwapColor(Handle&, _RGB(33, 66, 66), _RGB(0, 0, 132)) '       shadow color
        Handle& = SwapColor(Handle&, _RGB(0, 33, 255), _RGB(198, 33, 0)) '      gargoyle color

    ELSEIF LevNum = 3 THEN
        Handle& = SwapColor(Handle&, _RGB(0, 132, 132), _RGB(0, 148, 49))
        Handle& = SwapColor(Handle&, _RGB(0, 255, 255), _RGB(82, 247, 148))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 66), _RGB(0, 57, 16))
        Handle& = SwapColor(Handle&, _RGB(0, 33, 255), _RGB(198, 33, 0))

    ELSEIF LevNum = 4 THEN
        Handle& = SwapColor(Handle&, _RGB(0, 132, 132), _RGB(132, 115, 0))
        Handle& = SwapColor(Handle&, _RGB(0, 255, 255), _RGB(239, 189, 27))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 66), _RGB(57, 41, 0))
        Handle& = SwapColor(Handle&, _RGB(0, 33, 255), _RGB(24, 57, 239))

    ELSEIF LevNum = 5 THEN
        Handle& = SwapColor(Handle&, _RGB(0, 132, 132), _RGB(0, 165, 0))
        Handle& = SwapColor(Handle&, _RGB(0, 255, 255), _RGB(66, 198, 66))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 66), _RGB(0, 66, 0))
        Handle& = SwapColor(Handle&, _RGB(0, 33, 255), _RGB(198, 33, 0))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 255), _RGB(198, 33, 0)) '        water color

    ELSEIF LevNum = 6 THEN
        Handle& = SwapColor(Handle&, _RGB(0, 132, 132), _RGB(132, 132, 0))
        Handle& = SwapColor(Handle&, _RGB(0, 255, 255), _RGB(255, 198, 66))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 66), _RGB(66, 33, 0))
        Handle& = SwapColor(Handle&, _RGB(0, 33, 255), _RGB(198, 33, 0))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 255), _RGB(198, 33, 0))

    ELSEIF LevNum = 7 THEN
        Handle& = SwapColor(Handle&, _RGB(0, 132, 132), _RGB(0, 165, 0))
        Handle& = SwapColor(Handle&, _RGB(0, 255, 255), _RGB(66, 198, 66))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 66), _RGB(0, 66, 0))
        Handle& = SwapColor(Handle&, _RGB(0, 33, 255), _RGB(33, 66, 255))

    ELSEIF LevNum = 8 THEN
        Handle& = SwapColor(Handle&, _RGB(0, 132, 132), _RGB(132, 132, 132))
        Handle& = SwapColor(Handle&, _RGB(0, 255, 255), _RGB(255, 255, 255))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 66), _RGB(66, 66, 66))
        Handle& = SwapColor(Handle&, _RGB(0, 33, 255), _RGB(99, 165, 255))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 255), _RGB(99, 165, 255))

    ELSEIF LevNum = 9 THEN
        Handle& = SwapColor(Handle&, _RGB(0, 132, 132), _RGB(132, 132, 132))
        Handle& = SwapColor(Handle&, _RGB(0, 255, 255), _RGB(255, 255, 255))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 66), _RGB(66, 66, 66))
        Handle& = SwapColor(Handle&, _RGB(0, 33, 255), _RGB(66, 66, 66))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 255), _RGB(198, 33, 0))

    ELSEIF LevNum = 10 THEN
        Handle& = SwapColor(Handle&, _RGB(0, 132, 132), _RGB(180, 0, 0))
        Handle& = SwapColor(Handle&, _RGB(0, 255, 255), _RGB(255, 0, 0))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 66), _RGB(50, 0, 0))
        Handle& = SwapColor(Handle&, _RGB(0, 33, 255), _RGB(100, 0, 0))
        Handle& = SwapColor(Handle&, _RGB(255, 132, 0), _RGB(200, 0, 0))
        Handle& = SwapColor(Handle&, _RGB(33, 66, 255), _RGB(255, 0, 0))
    END IF
END SUB


'*********************************************************
'* PrintConsole
'*
'* Prints 'WhatToPrint$' to the debug console or main screen
'*********************************************************
SUB PrintConsole (WhatToPrint$, VerboseLevel)
    IF Settings.Debug <> 0 THEN _DEST _CONSOLE

    IF VerboseLevel = 1 THEN COLOR _RGB(128, 128, 128)
    IF VerboseLevel = 2 THEN COLOR _RGB(0, 255, 0)
    IF VerboseLevel = 3 THEN COLOR _RGB(255, 0, 0)

    PRINT WhatToPrint$
    _DEST Display&
END SUB



'*******************************************************************
'* Sub LoadDungTiles
'* Loads Misc tiles, like the blocks, stairs, sand, gargoyles, etc.
'*******************************************************************
SUB LoadDungTiles
    IF Settings.Debug <> 0 THEN
        CALL PrintConsole("Engine -> Loading Dungeon tile set:" + STR$(PalleteChange), 1)
    ELSE
        PRINT "Engine -> Loading Dungeon tile set:" + STR$(PalleteChange)
    END IF

    DungTile(1, 1) = _LOADIMAGE("Images\Block1.png")
    DungTile(2, 1) = _LOADIMAGE("Images\Garg1.png")
    DungTile(3, 1) = _LOADIMAGE("Images\Garg2.png")
    DungTile(4, 1) = _LOADIMAGE("Images\DWater.png")
    DungTile(5, 1) = _LOADIMAGE("Images\DDirt.png")
    DungTile(6, 1) = _LOADIMAGE("Images\DStairs.png")
    DungTile(7, 1) = _LOADIMAGE("Images\FloorSwitch.png")
    DungTile(8, 1) = _COPYIMAGE(DungTile(1, 1))
    DungTile(9, 1) = _COPYIMAGE(DungTile(6, 1))
    DungTile(10, 1) = _COPYIMAGE(DungTile(1, 1))
    DungTile(11, 1) = _COPYIMAGE(KeySprites(1))
    DungTile(12, 1) = _COPYIMAGE(KeySprites(1))
    DungTile(13, 1) = _LOADIMAGE("Images\Hole.png")
    DungTile(14, 1) = _LOADIMAGE("Images\DSand.png")
    DungTile(15, 1) = _COPYIMAGE(DungTile(2, 1))
    DungTile(16, 1) = _COPYIMAGE(DungTile(3, 1))


    ' Below changes to the different pallettes used in each dungeon
    FOR x = 1 TO 10
        FOR y = 2 TO 10
            DungTile(x, y) = _COPYIMAGE(DungTile(x, 1))
            CALL PalletteSwap(y, DungTile(x, y))
        NEXT y
    NEXT x

    FOR x = 15 TO 16
        FOR y = 2 TO 10
            DungTile(x, y) = _COPYIMAGE(DungTile(x, 1))
            CALL PalletteSwap(y, DungTile(x, y))
        NEXT y
    NEXT x

    FOR x = 11 TO 14
        FOR y = 2 TO 10
            DungTile(x, y) = _COPYIMAGE(DungTile(x, 1))
        NEXT y
    NEXT x

    CALL PalletteSwap(10, DungTile(14, 10))
END SUB

'*******************************************************************
'* Sub LoadMapSprites
'* Loads the door, main, and upper dungeon sprites and artwork
'*******************************************************************
SUB LoadMapSprites
    IF Settings.Debug <> 0 THEN
        CALL PrintConsole("Engine -> Loading door sprites", 1)
    ELSE
        PRINT "Engine -> Loading door sprites"
    END IF

    DungBackDrop(1, 1) = _LOADIMAGE("Images\DungMain.png")
    DungBackDrop(2, 1) = _LOADIMAGE("Images\DungTop.png")
    DungBackDrop(1, 1) = SwapColor(DungBackDrop(1, 1), _RGB(0, 0, 0), _RGB(1, 1, 1))
    DungBackDrop(2, 1) = SwapColor(DungBackDrop(2, 1), _RGB(0, 0, 0), _RGB(1, 1, 1))

    DungSprites(1, 1, 1) = _LOADIMAGE("Images\OpenN.png")
    DungSprites(1, 2, 1) = _LOADIMAGE("Images\OpenE.png")
    DungSprites(1, 3, 1) = _LOADIMAGE("Images\OpenS.png")
    DungSprites(1, 4, 1) = _LOADIMAGE("Images\OpenW.png")

    DungSprites(2, 1, 1) = _LOADIMAGE("Images\LockedN.png")
    DungSprites(2, 2, 1) = _LOADIMAGE("Images\LockedE.png")
    DungSprites(2, 3, 1) = _LOADIMAGE("Images\LockedS.png")
    DungSprites(2, 4, 1) = _LOADIMAGE("Images\LockedW.png")

    DungSprites(3, 1, 1) = _LOADIMAGE("Images\ClosedN.png")
    DungSprites(3, 2, 1) = _LOADIMAGE("Images\ClosedE.png")
    DungSprites(3, 3, 1) = _LOADIMAGE("Images\ClosedS.png")
    DungSprites(3, 4, 1) = _LOADIMAGE("Images\ClosedW.png")

    DungSprites(4, 1, 1) = _LOADIMAGE("Images\HoleN1.png")
    DungSprites(4, 2, 1) = _LOADIMAGE("Images\HoleE1.png")
    DungSprites(4, 3, 1) = _LOADIMAGE("Images\HoleS1.png")
    DungSprites(4, 4, 1) = _LOADIMAGE("Images\HoleW1.png")

    DungSprites(5, 1, 1) = _LOADIMAGE("Images\HoleN2.png")
    DungSprites(5, 2, 1) = _LOADIMAGE("Images\HoleE2.png")
    DungSprites(5, 3, 1) = _LOADIMAGE("Images\HoleS2.png")
    DungSprites(5, 4, 1) = _LOADIMAGE("Images\HoleW2.png")

    DungSprites(6, 1, 1) = _LOADIMAGE("Images\OpenNTop.png")
    DungSprites(6, 2, 1) = _LOADIMAGE("Images\OpenETop.png")
    DungSprites(6, 3, 1) = _LOADIMAGE("Images\OpenSTop.png")
    DungSprites(6, 4, 1) = _LOADIMAGE("Images\OpenWTop.png")

    ' Below changes to the different pallettes used in each dungeon
    FOR y = 2 TO 10
        FOR x = 1 TO 2
            DungBackDrop(x, y) = DungBackDrop(x, 1)
            CALL PalletteSwap(y, DungBackDrop(x, y))
        NEXT x
    NEXT y

    FOR z = 2 TO 10
        FOR y = 1 TO 4
            FOR x = 1 TO 6
                DungSprites(x, y, z) = DungSprites(x, y, 1)
                CALL PalletteSwap(z, DungSprites(x, y, z))
            NEXT x
        NEXT y
    NEXT z
END SUB




'*********************************************************
'*  LoadWorldTiles
'*
'*  Basically single sub call used to load world artwork
'*********************************************************
SUB LoadWorldTiles
    CALL LoadOverworldTiles
    CALL LoadDungTiles

    BasementBackdrop(1) = _LOADIMAGE("Images\Loot.png")
    BasementBackdrop(2) = _LOADIMAGE("Images\Tunnel.png")
END SUB


'$include:'ErrorDescription.bi'
'$include:'Maps.bi'
'$include:'Audio.bi'



