'*********************************************************
'* Maps.bi                                           6/12/21
'*
'* Contains all of the code to Load, Save, and manipulate the zones
'*
'*********************************************************




'*********************************************************
'* SaveMap
'*
'* Saves the active zone to its respected file
'*********************************************************
SUB SaveMap
    IF Settings.Debug <> 0 THEN CALL PrintConsole("World -> Saving Map: " + ActiveFile$, 1)

    IF LEFT$(ActiveFile$, 5) <> "World" THEN ActiveFile$ = "World\" + ActiveFile$

    OPEN ActiveFile$ FOR OUTPUT AS #1
    PRINT #1, ActiveMap.Type
    IF ActiveMap.Blackout = 1 THEN ActiveMap.SubType = ActiveMap.SubType + 9
    PRINT #1, ActiveMap.SubType
    PRINT #1, ActiveMap.BGM
    PRINT #1, ActiveMap.AmbientSound

    IF ActiveMap.NFile = "NULL" AND ActiveMap.Type = 2 THEN
        PRINT #1, 0;
    ELSEIF ActiveMap.Type = 2 THEN
        PRINT #1, ActiveMap.DoorN;
    END IF

    IF ActiveMap.EFile = "NULL" AND ActiveMap.Type = 2 THEN
        PRINT #1, 0;
    ELSEIF ActiveMap.Type = 2 THEN
        PRINT #1, ActiveMap.DoorE;
    END IF

    IF ActiveMap.SFile = "NULL" AND ActiveMap.Type = 2 THEN
        PRINT #1, 0;
    ELSEIF ActiveMap.Type = 2 THEN
        PRINT #1, ActiveMap.DoorS;
    END IF

    IF ActiveMap.WFile = "NULL" AND ActiveMap.Type = 2 THEN
        PRINT #1, 0
    ELSEIF ActiveMap.Type = 2 THEN
        PRINT #1, ActiveMap.DoorW
    END IF

    IF ActiveMap.Type = 2 THEN PRINT #1, ActiveMap.OpenCondition

    PRINT #1, ActiveMap.NFile
    PRINT #1, ActiveMap.EFile
    PRINT #1, ActiveMap.SFile
    PRINT #1, ActiveMap.WFile

    FOR y = 1 TO ActiveMap.SpriteLimitY
        FOR x = 1 TO ActiveMap.SpriteLimitX - 1
            PRINT #1, USING "### "; SpriteMatrix(x, y);
            'PRINT #1, SpriteMatrix(x, y);
        NEXT x
        PRINT #1, USING "### "; SpriteMatrix(ActiveMap.SpriteLimitX, y)
    NEXT y

    PRINT #1, ActiveMap.SecretZone
    PRINT #1, ActiveMap.Name
    CLOSE #1

    IF ActiveMap.Blackout = 1 THEN ActiveMap.SubType = ActiveMap.SubType - 9
END SUB





'*********************************************************
'* LoadMap
'*
'* Loads the zone data to a temp object then transfers it
'* File$ - File to load
'* TypePointer - Where to copy the temp to
'*               1 - Transfer to the active object game uses
'*               Anything else will sent to the object used for map \
'*               transitions.
'*********************************************************

SUB LoadMap (File$, TypePointer)
    DIM MapStruct AS Map
    IF Settings.Debug <> 0 THEN CALL PrintConsole("World -> Loading Map: " + File$, 1)

    IF LEFT$(File$, 5) <> "World" THEN File$ = "World\" + File$
    IF RIGHT$(File$, 4) <> ".msz" THEN File$ = File$ + ".msz"

    Settings.ErrorTracker = File$
    OPEN File$ FOR INPUT AS #1
    INPUT #1, MapStruct.Type
    INPUT #1, MapStruct.SubType
    INPUT #1, MapStruct.BGM
    INPUT #1, MapStruct.AmbientSound

    IF MapStruct.Type = 2 THEN
        INPUT #1, MapStruct.DoorN
        INPUT #1, MapStruct.DoorE
        INPUT #1, MapStruct.DoorS
        INPUT #1, MapStruct.DoorW
        INPUT #1, MapStruct.OpenCondition
    END IF

    INPUT #1, MapStruct.NFile
    INPUT #1, MapStruct.EFile
    INPUT #1, MapStruct.SFile
    INPUT #1, MapStruct.WFile


    IF MapStruct.Type = 1 OR MapStruct.Type > 2 THEN
        MapStruct.SpriteLimitX = 23
        MapStruct.SpriteLimitY = 13
    ELSE
        MapStruct.SpriteLimitX = 12
        MapStruct.SpriteLimitY = 7
    END IF

    CALL ActiveMapChange(MapStruct.Type, MapStruct.SpriteLimitX, MapStruct.SpriteLimitY)

    FOR y = 1 TO MapStruct.SpriteLimitY
        FOR x = 1 TO MapStruct.SpriteLimitX
            IF TypePointer = 1 THEN
                INPUT #1, SpriteMatrix(x, y)
            ELSE
                INPUT #1, TransMatrix(x, y)
            END IF
        NEXT x
    NEXT y

    INPUT #1, MapStruct.SecretZone
    INPUT #1, MapStruct.Name
    CLOSE #1

    Settings.MapActive = 1
    IF MapStruct.SubType > 9 THEN
        MapStruct.SubType = MapStruct.SubType - 9
        MapStruct.Blackout = 1
    END IF

    IF TypePointer = 1 THEN
        ActiveMap = MapStruct
        ActiveFile$ = File$
        IF ActiveMap.BGM = 4 THEN ActiveMap.BGM = 10
        CALL JukeBox(ActiveMap.BGM)
        CALL SoundBoard("", ActiveMap.AmbientSound)
    ELSE
        TransMap = MapStruct
    END IF
END SUB




'*********************************************************
'* SetMapSize
'*
'* sets the size of the tile matrix based on zone type
'* 2 - Dungeon
'* Everything else is overworld
'*********************************************************
SUB SetMapSize
    IF ActiveMap.Type = 1 OR ActiveMap.Type > 2 THEN
        ActiveMap.SpriteLimitX = 23
        ActiveMap.SpriteLimitY = 13
    ELSE
        ActiveMap.SpriteLimitX = 12
        ActiveMap.SpriteLimitY = 7
    END IF
END SUB




'*********************************************************
'* ActiveMapChange
'*
'* Called when a map or zone is being changed,
'* used to reset all variables needed to defaults or unloaded status
'*********************************************************

SUB ActiveMapChange (MapType, SpriteLimitX, SpriteLimitY)
    TIMER(WizzFireTimer) OFF
    Settings.ActiveSprite = 1
    Settings.LockDown = 0
    Settings.SoldItem = 0
    Settings.TextDone = 0
    Settings.SetupStore = 0
    Settings.Item1Cost = 0
    Settings.Item2Cost = 0
    Settings.Item3Cost = 0
    Settings.Item1Type = 0
    Settings.Item2Type = 0
    Settings.Item3Type = 0
    Player1.Clock = 0
    Player1.LockCandle = 0
    Player1.InRoom = 0
    Player1.LockSecondary = 0
    Player1.PlayedFlute = 0

    IF MapType = 1 OR MapType > 2 THEN
        Settings.TopLimit = 200
        Settings.LeftLimit = 0
        Settings.RightLimit = Settings.RezX - 1
        Settings.BottomLimit = Settings.RezY - 1
        Settings.TopSprite = Settings.OverworldTileArraySize + Settings.SpriteDivider
        Settings.SquareWidth = Player1.Size * 2
        Settings.SquareHeight = Player1.Size * 2
    ELSEIF MapType = 2 THEN
        Settings.TopLimit = 306
        Settings.LeftLimit = 123
        Settings.RightLimit = 892
        Settings.BottomLimit = 667
        Settings.TopSprite = Settings.DungTileArraySize + Settings.SpriteDivider
        Settings.SquareWidth = (Settings.RightLimit - Settings.LeftLimit) / SpriteLimitX
        Settings.SquareHeight = (Settings.BottomLimit - Settings.TopLimit) / SpriteLimitY
        Settings.ShowStairs = 0
    END IF
END SUB





'*********************************************************
'* LoadLinkMap
'*
'* Loads the zone to be modified based on actions in an adjacent zone
'* Afile$ - Zone file to load so it can be modified later
'* LinkFrom - The zone the player is currently in from the view
'*********************************************************
SUB LoadLinkMap (AFile$, LinkFrom)
    _DEST Display&
    File$ = AFile$
    IF UCASE$(RIGHT$(File$, 4)) <> Settings.MapExtension THEN File$ = RTRIM$(File$) + Settings.MapExtension
    File$ = "World\" + File$
    IF Settings.Debug <> 0 THEN CALL PrintConsole("World -> Loading map: " + AFile$ + " to link with", 1)
    IF Settings.EditorActive = 1 THEN PRINT "Linking file with: "; File$

    IF _FILEEXISTS(File$) THEN
        OPEN File$ FOR INPUT AS #1
        INPUT #1, LinkMap.Type
        INPUT #1, LinkMap.SubType
        INPUT #1, LinkMap.BGM
        INPUT #1, LinkMap.AmbientSound
        IF LinkMap.Type = 2 THEN
            INPUT #1, LinkMap.DoorN
            INPUT #1, LinkMap.DoorE
            INPUT #1, LinkMap.DoorS
            INPUT #1, LinkMap.DoorW
            INPUT #1, LinkMap.OpenCondition
        END IF
        INPUT #1, LinkMap.NFile
        INPUT #1, LinkMap.EFile
        INPUT #1, LinkMap.SFile
        INPUT #1, LinkMap.WFile

        IF LinkMap.Type = 1 THEN
            LinkMap.SpriteLimitX = 23
            LinkMap.SpriteLimitY = 13
        ELSEIF LinkMap.Type = 2 THEN
            LinkMap.SpriteLimitX = 12
            LinkMap.SpriteLimitY = 7
        END IF

        FOR y = 1 TO LinkMap.SpriteLimitY
            FOR x = 1 TO LinkMap.SpriteLimitX
                INPUT #1, LinkSpriteMatrix(x, y)
            NEXT x
        NEXT y

        INPUT #1, LinkMap.SecretZone
        INPUT #1, LinkMap.Name
        CLOSE #1

        IF ActiveMap.Type <> LinkMap.Type THEN
            IF Settings.EditorActive = 1 THEN PRINT "Map types do not match. Aborting link with: "; File$
            EXIT SUB
        END IF

    ELSE
        IF Settings.EditorActive = 1 THEN PRINT "File does not exist, creating data for new file."
        LinkMap.Type = ActiveMap.Type
        LinkMap.SubType = ActiveMap.SubType
        LinkMap.BGM = ActiveMap.BGM
        LinkMap.AmbientSound = 0

        IF LinkMap.Type = 2 THEN
            LinkMap.DoorN = 0
            LinkMap.DoorE = 0
            LinkMap.DoorS = 0
            LinkMap.DoorW = 0
            LinkMap.OpenCondition = 0
        END IF

        LinkMap.NFile = "NULL"
        LinkMap.EFile = "NULL"
        LinkMap.SFile = "NULL"
        LinkMap.WFile = "NULL"

        IF LinkMap.Type = 1 OR LinkMap.Type > 2 THEN
            LinkMap.SpriteLimitX = 23
            LinkMap.SpriteLimitY = 13
        ELSEIF LinkMap.Type = 2 THEN
            LinkMap.SpriteLimitX = 12
            LinkMap.SpriteLimitY = 7
        END IF

        FOR y = 1 TO LinkMap.SpriteLimitY
            FOR x = 1 TO LinkMap.SpriteLimitX
                LinkSpriteMatrix(x, y) = 0
            NEXT x
        NEXT y
        LinkMap.SecretZone = "NULL"
        LinkMap.Name = "NULL"

        IF Settings.EditorActive = 1 THEN PRINT " Creating: " + File$
    END IF


    IF LinkFrom = 1 AND LinkMap.Type = 1 AND LinkMap.Type = ActiveMap.Type THEN
        FOR x = 1 TO LinkMap.SpriteLimitX
            LinkSpriteMatrix(x, LinkMap.SpriteLimitY) = SpriteMatrix(x, 1)
        NEXT x
    ELSEIF LinkFrom = 2 AND LinkMap.Type = 1 AND LinkMap.Type = ActiveMap.Type THEN
        FOR y = 1 TO LinkMap.SpriteLimitY
            LinkSpriteMatrix(1, y) = SpriteMatrix(LinkMap.SpriteLimitX, y)
        NEXT y
    ELSEIF LinkFrom = 3 AND LinkMap.Type = 1 AND LinkMap.Type = ActiveMap.Type THEN
        FOR x = 1 TO LinkMap.SpriteLimitX
            LinkSpriteMatrix(x, 1) = SpriteMatrix(x, LinkMap.SpriteLimitY)
        NEXT x
    ELSEIF LinkFrom = 4 AND LinkMap.Type = 1 AND LinkMap.Type = ActiveMap.Type THEN
        FOR y = 1 TO LinkMap.SpriteLimitY
            LinkSpriteMatrix(LinkMap.SpriteLimitX, y) = SpriteMatrix(1, y)
        NEXT y
    END IF
END SUB




'*********************************************************
'* LinkMaps
'*
'* Will be called when a zone is modified in such a way it
'* affects the adjacent zones. Will update those zones and
'* save those changes. Ex: A bomb blowing a hole in the wall
'*
'* Afile$ - Zone file to output to
'* LinkFrom - The zone the player is currently in from the view
'*            of the zone being modified 
'*            1 - North then rotate clockwise for 2-4
'* DoorOveride - The door code to override with
'*               1 - Open (door on other side ahs been unlocked)
'*               5 - Wall that has been bombed on other side   
'*********************************************************

SUB LinkMaps (AFile$, LinkFrom, DoorOverride)
    IF LEFT$(ActiveFile$, 5) <> "World" THEN ActiveFile$ = "World\" + ActiveFile$
    File$ = AFile$
    IF UCASE$(RIGHT$(File$, 4)) <> Settings.MapExtension THEN File$ = RTRIM$(File$) + Settings.MapExtension
    File$ = "World\" + File$

    OPEN File$ FOR OUTPUT AS #1
    PRINT #1, LinkMap.Type
    PRINT #1, LinkMap.SubType
    PRINT #1, LinkMap.BGM
    PRINT #1, LinkMap.AmbientSound

    IF LinkFrom = 1 THEN
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.DoorN;
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.DoorE;
        IF LinkMap.Type = 2 THEN PRINT #1, DoorOverride;
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.DoorW
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.OpenCondition
        PRINT #1, LinkMap.NFile
        PRINT #1, LinkMap.EFile
        PRINT #1, MID$(ActiveFile$, 7, LEN(ActiveFile$) - 10)
        PRINT #1, LinkMap.WFile

    ELSEIF LinkFrom = 2 THEN
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.DoorN;
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.DoorE;
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.DoorS;
        IF LinkMap.Type = 2 THEN PRINT #1, DoorOverride
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.OpenCondition
        PRINT #1, LinkMap.NFile
        PRINT #1, LinkMap.EFile
        PRINT #1, LinkMap.SFile
        PRINT #1, MID$(ActiveFile$, 7, LEN(ActiveFile$) - 10)

    ELSEIF LinkFrom = 3 THEN
        IF LinkMap.Type = 2 THEN PRINT #1, DoorOverride;
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.DoorE;
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.DoorS;
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.DoorW
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.OpenCondition
        PRINT #1, MID$(ActiveFile$, 7, LEN(ActiveFile$) - 10)
        PRINT #1, LinkMap.EFile
        PRINT #1, LinkMap.SFile
        PRINT #1, LinkMap.WFile

    ELSEIF LinkFrom = 4 THEN
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.DoorN;
        IF LinkMap.Type = 2 THEN PRINT #1, DoorOverride;
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.DoorS;
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.DoorW
        IF LinkMap.Type = 2 THEN PRINT #1, LinkMap.OpenCondition
        PRINT #1, LinkMap.NFile
        PRINT #1, MID$(ActiveFile$, 7, LEN(ActiveFile$) - 10)
        PRINT #1, LinkMap.SFile
        PRINT #1, LinkMap.WFile
    END IF

    FOR y = 1 TO LinkMap.SpriteLimitY
        FOR x = 1 TO LinkMap.SpriteLimitX - 1
            PRINT #1, USING "### "; LinkSpriteMatrix(x, y);
        NEXT x
        PRINT #1, USING "### "; LinkSpriteMatrix(LinkMap.SpriteLimitX, y)
    NEXT y

    PRINT #1, LinkMap.SecretZone

    IF RTRIM$(LinkMap.Name) <> "NULL" THEN
        PRINT #1, LinkMap.Name
    ELSE
        PRINT #1, ActiveMap.Name
    END IF
    CLOSE #1

    IF Settings.EditorActive = 1 THEN PRINT "Done"
END SUB



'*******************************************************************
'* Sub DrawMap
'* draws sprites to specefied screens
'*
'* Target& - Screen that main chunk of map is drawn to
'* ForeTarget - Screen used to draw top part of maps. ex overpasses etc.
'* TargetSwitch - used to select wich matrix to read data from
'*                1 reads from transition matrix and object, usually after a transition has been completed
'*                2 reads from main matrix and map object
'*********************************************************************
SUB DrawMap (Target&, ForeTarget&, TargetSwitch, DrawBlocks)
    DIM TargetMatrix(24, 14)
    DIM MapStruct AS Map

    FOR x = 1 TO 24
        FOR y = 1 TO 14
            IF TargetSwitch = 1 THEN
                TargetMatrix(x, y) = TransMatrix(x, y)
                MapStruct = TransMap
            ELSE
                TargetMatrix(x, y) = SpriteMatrix(x, y)
                MapStruct = ActiveMap
            END IF
        NEXT y
    NEXT x

    IF Player1.Health <= 0 AND MapStruct.Type = 2 THEN MapStruct.SubType = 10

    _DEST ForeTarget&
    CLS
    _CLEARCOLOR _RGB(0, 0, 0)
    _DEST Target&
    CLS
    _CLEARCOLOR _RGB(0, 0, 0)

    IF MapStruct.Type = 1 OR MapStruct.Type > 2 THEN 'if map is overworld or cave or basement
        IF MapStruct.Type = 1 AND MapStruct.SubType = 1 THEN
            IF Player1.Health <= 0 THEN
                LINE (0, 200)-(Settings.RezX, Settings.RezY), _RGB(255, 0, 0), BF 'overworld ground when link is dead
            ELSE
                _PUTIMAGE (0, 200)-(Settings.RezX, Settings.RezY), OverworldTile(19), Target& 'overworld ground
            END IF

        ELSEIF MapStruct.Type = 1 AND MapStruct.SubType = 2 THEN
            LINE (0, 200)-(Settings.RezX, Settings.RezY), _RGB(122, 122, 122), BF 'cemetary ground

        ELSEIF MapStruct.Type = 5 THEN
            _PUTIMAGE (0, 200)-(Settings.RezX, Settings.RezY), BasementBackdrop(MapStruct.SubType), Target& 'basement background


        ELSE
            LINE (0, 200)-(Settings.RezX, Settings.RezY), _RGB(0, 26, 255), BF 'Open Sea
        END IF

        IF MapStruct.Type = 3 OR MapStruct.Type = 4 THEN LINE (0, 200)-(Settings.RezX, Settings.RezY), _RGB(1, 1, 1), BF 'cave ground


        FOR y = 1 TO MapStruct.SpriteLimitY 'loop draws map sprites off info from the matrix
            FOR x = 1 TO MapStruct.SpriteLimitX
                IF TargetMatrix(x, y) > Settings.SpriteDivider THEN
                    Settings.ErrorTracker = "Drawing Tile#:" + STR$(TargetMatrix(x, y))

                    SpriteWidth = (Settings.SquareWidth / 2) - 1
                    SpriteHeight = (Settings.SquareHeight / 2) - 1
                    SpriteX = (Settings.SquareWidth * (x - 1)) + Settings.LeftLimit + SpriteWidth
                    Spritey = (Settings.SquareHeight * (y - 1)) + Settings.TopLimit + SpriteHeight

                    IF TargetMatrix(x, y) >= Settings.SpriteDivider + 97 AND TargetMatrix(x, y) <= Settings.SpriteDivider + 106 THEN
                        FalseLake = TargetMatrix(x, y) - 125 '52
                        IF FalseLake = 61 THEN FalseLake = 56
                        _PUTIMAGE (SpriteX - SpriteWidth - 1, Spritey - SpriteHeight - 1)-(SpriteX + SpriteWidth + 1, Spritey + SpriteHeight + 1), OverworldTile(FalseLake), Target&
                    ELSEIF TargetMatrix(x, y) <> Settings.SpriteDivider + 107 THEN
                        _PUTIMAGE (SpriteX - SpriteWidth - 1, Spritey - SpriteHeight - 1)-(SpriteX + SpriteWidth + 1, Spritey + SpriteHeight + 1), OverworldTile(TargetMatrix(x, y) - Settings.SpriteDivider), Target&
                    END IF

                ELSEIF TargetMatrix(x, y) = 32 OR TargetMatrix(x, y) = 71 THEN
                    SpriteWidth = (Settings.SquareWidth / 2) - 1
                    SpriteHeight = (Settings.SquareHeight / 2) - 1
                    SpriteX = (Settings.SquareWidth * (x - 1)) + Settings.LeftLimit + SpriteWidth
                    Spritey = (Settings.SquareHeight * (y - 1)) + Settings.TopLimit + SpriteHeight

                    _PUTIMAGE (SpriteX - SpriteWidth - 1, Spritey - SpriteHeight - 1)-(SpriteX + SpriteWidth + 1, Spritey + SpriteHeight + 1), WaterfallSprites(1), Target&
                END IF
            NEXT x
        NEXT y
        _CLEARCOLOR _RGB(0, 26, 255), Target& 

    ELSEIF MapStruct.Type = 2 THEN 'if map is dungeon

        _PUTIMAGE (0, 200)-(Settings.RezX, Settings.RezY), DungBackDrop(1, MapStruct.SubType), Target& 'draws floor and bottom half of walls
        _PUTIMAGE (0, 200)-(Settings.RezX, Settings.RezY), DungBackDrop(2, MapStruct.SubType), ForeTarget& 'draws top half of wall and overpasses

        IF MapStruct.DoorN <> 0 THEN CALL DrawDoors(443, 242, 573, 306, MapStruct.DoorN, 1, Target&, ForeTarget&, MapStruct.SubType) 'north room exit
        IF MapStruct.DoorE <> 0 THEN CALL DrawDoors(892, 435, 973, 540, MapStruct.DoorE, 2, Target&, ForeTarget&, MapStruct.SubType) 'east room exit
        IF MapStruct.DoorS <> 0 THEN CALL DrawDoors(443, 668, 573, 732, MapStruct.DoorS, 3, Target&, ForeTarget&, MapStruct.SubType) 'south room exit
        IF MapStruct.DoorW <> 0 THEN CALL DrawDoors(44, 435, 123, 540, MapStruct.DoorW, 4, Target&, ForeTarget&, MapStruct.SubType) 'west room exit

        SpriteWidth = (Settings.SquareWidth / 2) - 1
        SpriteHeight = (Settings.SquareHeight / 2) - 1

        FOR y = 1 TO MapStruct.SpriteLimitY 'draws map sprites
            FOR x = 1 TO MapStruct.SpriteLimitX
                IF TargetMatrix(x, y) > Settings.SpriteDivider AND TargetMatrix(x, y) < Settings.SpriteDivider + 8 OR TargetMatrix(x, y) = Settings.SpriteDivider + 14 OR TargetMatrix(x, y) = Settings.SpriteDivider + 13 THEN

                    IF x = 1 AND y = 1 THEN
                        IF TargetMatrix(x, y) = Settings.SpriteDivider + 14 OR TargetMatrix(x, y) = Settings.SpriteDivider + 13 OR TargetMatrix(x, y) = Settings.SpriteDivider + 5 THEN CALL RepeatTile(TargetMatrix(x, y), MapStruct.SubType, MapStruct.SpriteLimitX, MapStruct.SpriteLimitY, Target&)
                    END IF

                    Settings.ErrorTracker = "Drawing Tile#:" + STR$(TargetMatrix(x, y))

                    SpriteX = (Settings.SquareWidth * (x - 1)) + Settings.LeftLimit + SpriteWidth
                    Spritey = (Settings.SquareHeight * (y - 1)) + Settings.TopLimit + SpriteHeight

                    _PUTIMAGE (SpriteX - SpriteWidth - 1, Spritey - SpriteHeight - 1)-(SpriteX + SpriteWidth + 1, Spritey + SpriteHeight + 1), DungTile(TargetMatrix(x, y) - Settings.SpriteDivider, MapStruct.SubType), Target&

                ELSEIF TargetMatrix(x, y) = Settings.SpriteDivider + 8 OR TargetMatrix(x, y) = Settings.SpriteDivider + 10 THEN
                    IF DrawBlocks = 1 THEN
                        SpriteX = (Settings.SquareWidth * (x - 1)) + Settings.LeftLimit + SpriteWidth
                        Spritey = (Settings.SquareHeight * (y - 1)) + Settings.TopLimit + SpriteHeight

                        _PUTIMAGE (SpriteX - SpriteWidth - 1, Spritey - SpriteHeight - 1)-(SpriteX + SpriteWidth + 1, Spritey + SpriteHeight + 1), DungTile(1, MapStruct.SubType), Target&
                    END IF
                END IF
            NEXT x
        NEXT y
    END IF
END SUB



'******************************************************************************
'* DrawDoors - Used to draw doors and to which screen
'*
'* X1 & Y1 are top left point of sprite to be drawn
'* X2 & Y2 is the bottom right point of sprite to be drawn
'* Door types:  0 = Solid Wall
'*              1 = Open Door
'*              2 = Locked door
'*              3 = Auto door (opens when room cleared)
'*              4 = Bombable wall
'*              5 = Wall that has been bombed
'*              6 = Secret passage
'*              7 = One way secret passage
'*
'* SpritePointer: 1 = North, 2 = East, 3 = South, 4 = West
'* Target: Screen to draw main chunk of sprite
'* ForeTarget: Screen used to draw upper part of door to give illusion of an overpass
'*****************************************************************************************
SUB DrawDoors (X1, Y1, X2, Y2, DoorType, SpritePointer, Target&, ForeTarget&, MapPallet)
    IF DoorType = 1 THEN
        _PUTIMAGE (X1, Y1)-(X2, Y2), DungSprites(1, SpritePointer, MapPallet), Target&
        _PUTIMAGE (X1, Y1)-(X2, Y2), DungSprites(6, SpritePointer, MapPallet), ForeTarget&

    ELSEIF DoorType = 2 THEN
        _PUTIMAGE (X1, Y1)-(X2, Y2), DungSprites(2, SpritePointer, MapPallet), Target&

    ELSEIF DoorType = 3 THEN
        IF Settings.EditorActive = 1 OR Settings.LockDown = 1 THEN
            _PUTIMAGE (X1, Y1)-(X2, Y2), DungSprites(3, SpritePointer, MapPallet), Target&
        ELSE
            _PUTIMAGE (X1, Y1)-(X2, Y2), DungSprites(1, SpritePointer, MapPallet), Target&
            _PUTIMAGE (X1, Y1)-(X2, Y2), DungSprites(6, SpritePointer, MapPallet), ForeTarget&
        END IF

    ELSEIF DoorType = 4 AND Settings.EditorActive = 1 THEN
        _PUTIMAGE (X1, Y1)-(X2, Y2), DungSprites(4, SpritePointer, MapPallet), Target&
        _PUTIMAGE (X1, Y1)-(X2, Y2), DungSprites(5, SpritePointer, MapPallet), Target&

    ELSEIF DoorType = 5 THEN
        IF Settings.EditorActive = 1 THEN
            _PUTIMAGE (X1, Y1)-(X2, Y2), DungSprites(4, SpritePointer, MapPallet), Target&
            _PUTIMAGE (X1, Y1)-(X2, Y2), DungSprites(5, SpritePointer, MapPallet), Target&
            _DEST Target&
            LINE (X1, Y1)-(X2, Y2), _RGB(255, 0, 0), B
        ELSE
            _PUTIMAGE (X1, Y1)-(X2, Y2), DungSprites(4, SpritePointer, MapPallet), Target&
            _PUTIMAGE (X1, Y1)-(X2, Y2), DungSprites(5, SpritePointer, MapPallet), ForeTarget&
        END IF

    ELSEIF DoorType = 6 AND Settings.EditorActive = 1 THEN
        _DEST Target&
        LINE (X1, Y1)-(X2, Y2), _RGB(0, 255, 0), B

    ELSEIF DoorType = 7 AND Settings.EditorActive = 1 THEN
        _DEST Target&
        LINE (X1, Y1)-(X2, Y2), _RGB(255, 0, 0), B
    END IF
END SUB








